package gw.spec.core.expressions.arithmetic.addition
uses java.lang.Byte
uses java.lang.Character
uses java.lang.Integer
uses java.lang.Short
uses java.lang.Long
uses java.lang.Float
uses java.lang.Double
uses java.math.BigInteger
uses java.math.BigDecimal
uses gw.spec.core.expressions.arithmetic.ArithmeticTestBase

class Addition_FloatTest extends ArithmeticTestBase {

  function testFloatPByteAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_byte(0))
    assertEquals(b_float(23.0), b_float(0.0) + p_byte(23))
    assertEquals(b_float(-32.0), b_float(0.0) + p_byte(-32))
    assertEquals(b_float(127.0), b_float(0.0) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-128.0), b_float(0.0) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_byte(0))
    assertEquals(b_float(46.0), b_float(23.0) + p_byte(23))
    assertEquals(b_float(-9.0), b_float(23.0) + p_byte(-32))
    assertEquals(b_float(150.0), b_float(23.0) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-105.0), b_float(23.0) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_byte(0))
    assertEquals(b_float(46.123), b_float(23.123) + p_byte(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + p_byte(-32))
    assertEquals(b_float(150.123), b_float(23.123) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-104.877), b_float(23.123) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_byte(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_byte(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + p_byte(-32))
    assertEquals(b_float(95.0), b_float(-32.0) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-160.0), b_float(-32.0) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_byte(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_byte(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + p_byte(-32))
    assertEquals(b_float(94.544), b_float(-32.456) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-160.456), b_float(-32.456) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_byte(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_byte(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_byte(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_byte(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_byte(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_byte(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_byte(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_byte(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_byte(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_byte(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_byte(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_byte(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_byte(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_byte(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + p_byte(-32))
    assertEquals(b_float(127.0), b_float(Float.MIN_VALUE) + p_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-128.0), b_float(Float.MIN_VALUE) + p_byte(Byte.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_byte(0)))
  }

  function testFloatByteAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_byte(0))
    assertEquals(b_float(23.0), b_float(0.0) + b_byte(23))
    assertEquals(b_float(-32.0), b_float(0.0) + b_byte(-32))
    assertEquals(b_float(127.0), b_float(0.0) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-128.0), b_float(0.0) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_byte(0))
    assertEquals(b_float(46.0), b_float(23.0) + b_byte(23))
    assertEquals(b_float(-9.0), b_float(23.0) + b_byte(-32))
    assertEquals(b_float(150.0), b_float(23.0) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-105.0), b_float(23.0) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_byte(0))
    assertEquals(b_float(46.123), b_float(23.123) + b_byte(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + b_byte(-32))
    assertEquals(b_float(150.123), b_float(23.123) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-104.877), b_float(23.123) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_byte(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_byte(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + b_byte(-32))
    assertEquals(b_float(95.0), b_float(-32.0) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-160.0), b_float(-32.0) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_byte(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_byte(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + b_byte(-32))
    assertEquals(b_float(94.544), b_float(-32.456) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-160.456), b_float(-32.456) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_byte(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_byte(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_byte(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_byte(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_byte(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_byte(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_byte(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_byte(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_byte(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_byte(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_byte(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_byte(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_byte(Byte.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_byte(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_byte(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + b_byte(-32))
    assertEquals(b_float(127.0), b_float(Float.MIN_VALUE) + b_byte(Byte.MAX_VALUE))
    assertEquals(b_float(-128.0), b_float(Float.MIN_VALUE) + b_byte(Byte.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_byte(0)))
  }

  function testFloatPShortAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_short(0))
    assertEquals(b_float(23.0), b_float(0.0) + p_short(23))
    assertEquals(b_float(-32.0), b_float(0.0) + p_short(-32))
    assertEquals(b_float(32767.0), b_float(0.0) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32768.0), b_float(0.0) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_short(0))
    assertEquals(b_float(46.0), b_float(23.0) + p_short(23))
    assertEquals(b_float(-9.0), b_float(23.0) + p_short(-32))
    assertEquals(b_float(32790.0), b_float(23.0) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32745.0), b_float(23.0) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_short(0))
    assertEquals(b_float(46.123), b_float(23.123) + p_short(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + p_short(-32))
    assertEquals(b_float(32790.12), b_float(23.123) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32744.877), b_float(23.123) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_short(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_short(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + p_short(-32))
    assertEquals(b_float(32735.0), b_float(-32.0) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32800.0), b_float(-32.0) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_short(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_short(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + p_short(-32))
    assertEquals(b_float(32734.545), b_float(-32.456) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32800.457), b_float(-32.456) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_short(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_short(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_short(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_short(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_short(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_short(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_short(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_short(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_short(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_short(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_short(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_short(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_short(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_short(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + p_short(-32))
    assertEquals(b_float(32767.0), b_float(Float.MIN_VALUE) + p_short(Short.MAX_VALUE))
    assertEquals(b_float(-32768.0), b_float(Float.MIN_VALUE) + p_short(Short.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_short(0)))
  }

  function testFloatShortAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_short(0))
    assertEquals(b_float(23.0), b_float(0.0) + b_short(23))
    assertEquals(b_float(-32.0), b_float(0.0) + b_short(-32))
    assertEquals(b_float(32767.0), b_float(0.0) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32768.0), b_float(0.0) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_short(0))
    assertEquals(b_float(46.0), b_float(23.0) + b_short(23))
    assertEquals(b_float(-9.0), b_float(23.0) + b_short(-32))
    assertEquals(b_float(32790.0), b_float(23.0) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32745.0), b_float(23.0) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_short(0))
    assertEquals(b_float(46.123), b_float(23.123) + b_short(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + b_short(-32))
    assertEquals(b_float(32790.12), b_float(23.123) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32744.877), b_float(23.123) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_short(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_short(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + b_short(-32))
    assertEquals(b_float(32735.0), b_float(-32.0) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32800.0), b_float(-32.0) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_short(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_short(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + b_short(-32))
    assertEquals(b_float(32734.545), b_float(-32.456) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32800.457), b_float(-32.456) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_short(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_short(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_short(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_short(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_short(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_short(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_short(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_short(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_short(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_short(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_short(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_short(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_short(Short.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_short(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_short(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + b_short(-32))
    assertEquals(b_float(32767.0), b_float(Float.MIN_VALUE) + b_short(Short.MAX_VALUE))
    assertEquals(b_float(-32768.0), b_float(Float.MIN_VALUE) + b_short(Short.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_short(0)))
  }

  function testFloatPCharAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_char(0))
    assertEquals(b_float(23.0), b_float(0.0) + p_char(23))
    assertEquals(b_float(65535.0), b_float(0.0) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_char(0))
    assertEquals(b_float(46.0), b_float(23.0) + p_char(23))
    assertEquals(b_float(65558.0), b_float(23.0) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_char(0))
    assertEquals(b_float(46.123), b_float(23.123) + p_char(23))
    assertEquals(b_float(65558.125), b_float(23.123) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_char(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_char(23))
    assertEquals(b_float(65503.0), b_float(-32.0) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_char(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_char(23))
    assertEquals(b_float(65502.543), b_float(-32.456) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_char(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_char(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_char(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_char(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_char(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_char(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_char(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_char(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_char(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_char(23))
    assertEquals(b_float(65535.0), b_float(Float.MIN_VALUE) + p_char(Character.MAX_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_char(0)))
  }

  function testFloatCharacterAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_char(0))
    assertEquals(b_float(23.0), b_float(0.0) + b_char(23))
    assertEquals(b_float(65535.0), b_float(0.0) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_char(0))
    assertEquals(b_float(46.0), b_float(23.0) + b_char(23))
    assertEquals(b_float(65558.0), b_float(23.0) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_char(0))
    assertEquals(b_float(46.123), b_float(23.123) + b_char(23))
    assertEquals(b_float(65558.125), b_float(23.123) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_char(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_char(23))
    assertEquals(b_float(65503.0), b_float(-32.0) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_char(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_char(23))
    assertEquals(b_float(65502.543), b_float(-32.456) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_char(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_char(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_char(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_char(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_char(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_char(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_char(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_char(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_char(Character.MAX_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_char(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_char(23))
    assertEquals(b_float(65535.0), b_float(Float.MIN_VALUE) + b_char(Character.MAX_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_char(0)))
  }

  function testFloatPIntAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_int(0))
    assertEquals(b_float(23.0), b_float(0.0) + p_int(23))
    assertEquals(b_float(-32.0), b_float(0.0) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(0.0) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(0.0) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_int(0))
    assertEquals(b_float(46.0), b_float(23.0) + p_int(23))
    assertEquals(b_float(-9.0), b_float(23.0) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(23.0) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(23.0) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_int(0))
    assertEquals(b_float(46.123), b_float(23.123) + p_int(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(23.123) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(23.123) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_int(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_int(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(-32.0) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(-32.0) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_int(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_int(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(-32.456) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(-32.456) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_int(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_int(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_int(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_int(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_int(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_int(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_int(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_int(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_int(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_int(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_int(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_int(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_int(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + p_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(Float.MIN_VALUE) + p_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(Float.MIN_VALUE) + p_int(Integer.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_int(0)))
  }

  function testFloatIntegerAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_int(0))
    assertEquals(b_float(23.0), b_float(0.0) + b_int(23))
    assertEquals(b_float(-32.0), b_float(0.0) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(0.0) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(0.0) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_int(0))
    assertEquals(b_float(46.0), b_float(23.0) + b_int(23))
    assertEquals(b_float(-9.0), b_float(23.0) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(23.0) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(23.0) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_int(0))
    assertEquals(b_float(46.123), b_float(23.123) + b_int(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(23.123) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(23.123) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_int(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_int(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(-32.0) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(-32.0) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_int(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_int(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(-32.456) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(-32.456) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_int(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_int(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_int(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_int(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_int(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_int(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_int(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_int(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_int(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_int(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_int(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_int(Integer.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_int(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_int(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + b_int(-32))
    assertEquals(b_float(Float.parseFloat("2.14748365E9")), b_float(Float.MIN_VALUE) + b_int(Integer.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-2.14748365E9")), b_float(Float.MIN_VALUE) + b_int(Integer.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_int(0)))
  }

  function testFloatPLongAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_long(0))
    assertEquals(b_float(23.0), b_float(0.0) + p_long(23))
    assertEquals(b_float(-32.0), b_float(0.0) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(0.0) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(0.0) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_long(0))
    assertEquals(b_float(46.0), b_float(23.0) + p_long(23))
    assertEquals(b_float(-9.0), b_float(23.0) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(23.0) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(23.0) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_long(0))
    assertEquals(b_float(46.123), b_float(23.123) + p_long(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(23.123) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(23.123) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_long(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_long(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(-32.0) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(-32.0) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_long(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_long(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(-32.456) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(-32.456) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_long(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_long(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_long(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_long(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_long(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_long(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_long(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_long(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_long(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_long(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_long(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_long(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_long(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + p_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(Float.MIN_VALUE) + p_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(Float.MIN_VALUE) + p_long(Long.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_long(0)))
  }

  function testFloatLongAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_long(0))
    assertEquals(b_float(23.0), b_float(0.0) + b_long(23))
    assertEquals(b_float(-32.0), b_float(0.0) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(0.0) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(0.0) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_long(0))
    assertEquals(b_float(46.0), b_float(23.0) + b_long(23))
    assertEquals(b_float(-9.0), b_float(23.0) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(23.0) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(23.0) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_long(0))
    assertEquals(b_float(46.123), b_float(23.123) + b_long(23))
    assertEquals(b_float(-8.877001), b_float(23.123) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(23.123) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(23.123) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_long(0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_long(23))
    assertEquals(b_float(-64.0), b_float(-32.0) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(-32.0) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(-32.0) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_long(0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_long(23))
    assertEquals(b_float(-64.456), b_float(-32.456) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(-32.456) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(-32.456) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_long(0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_long(23))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_long(-32))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_long(0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_long(23))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_long(-32))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_long(0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_long(23))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_long(-32))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_long(0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_long(23))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_long(Long.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_long(0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_long(23))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + b_long(-32))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), b_float(Float.MIN_VALUE) + b_long(Long.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), b_float(Float.MIN_VALUE) + b_long(Long.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_long(0)))
  }

  function testFloatPFloatAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + p_float(0.0))
    assertEquals(b_float(23.0), b_float(0.0) + p_float(23.0))
    assertEquals(b_float(23.123), b_float(0.0) + p_float(23.123))
    assertEquals(b_float(-32.0), b_float(0.0) + p_float(-32.0))
    assertEquals(b_float(-32.456), b_float(0.0) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(0.0) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(0.0) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(0.0) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(0.0) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(0.0) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + p_float(0.0))
    assertEquals(b_float(46.0), b_float(23.0) + p_float(23.0))
    assertEquals(b_float(46.123), b_float(23.0) + p_float(23.123))
    assertEquals(b_float(-9.0), b_float(23.0) + p_float(-32.0))
    assertEquals(b_float(-9.456001), b_float(23.0) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(23.0) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(23.0) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(23.0) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(23.0) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(23.0), b_float(23.0) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + p_float(0.0))
    assertEquals(b_float(46.123), b_float(23.123) + p_float(23.0))
    assertEquals(b_float(46.246), b_float(23.123) + p_float(23.123))
    assertEquals(b_float(-8.877001), b_float(23.123) + p_float(-32.0))
    assertEquals(b_float(-9.333002), b_float(23.123) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(23.123) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(23.123) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(23.123) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(23.123) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(23.123), b_float(23.123) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + p_float(0.0))
    assertEquals(b_float(-9.0), b_float(-32.0) + p_float(23.0))
    assertEquals(b_float(-8.877001), b_float(-32.0) + p_float(23.123))
    assertEquals(b_float(-64.0), b_float(-32.0) + p_float(-32.0))
    assertEquals(b_float(-64.456), b_float(-32.0) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(-32.0) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(-32.0) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(-32.0) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(-32.0) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(-32.0), b_float(-32.0) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + p_float(0.0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + p_float(23.0))
    assertEquals(b_float(-9.333002), b_float(-32.456) + p_float(23.123))
    assertEquals(b_float(-64.456), b_float(-32.456) + p_float(-32.0))
    assertEquals(b_float(-64.912), b_float(-32.456) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(-32.456) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(-32.456) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(-32.456) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(-32.456) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(-32.456), b_float(-32.456) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(0.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(23.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(23.123))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(-32.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.POSITIVE_INFINITY) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.POSITIVE_INFINITY) + p_float(Float.NaN_))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(0.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(23.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(23.123))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(-32.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(-32.456))
    assertEquals(b_float(Float.NaN_), b_float(Float.NEGATIVE_INFINITY) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NEGATIVE_INFINITY) + p_float(Float.NaN_))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(0.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(23.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(23.123))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(-32.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(-32.456))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(Float.NaN_))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(0.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(23.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(23.123))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(-32.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.MAX_VALUE) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.MAX_VALUE) + p_float(Float.NaN_))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + p_float(0.0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + p_float(23.0))
    assertEquals(b_float(23.123), b_float(Float.MIN_VALUE) + p_float(23.123))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + p_float(-32.0))
    assertEquals(b_float(-32.456), b_float(Float.MIN_VALUE) + p_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MIN_VALUE) + p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.MIN_VALUE) + p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.MIN_VALUE) + p_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MIN_VALUE) + p_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("2.8E-45")), b_float(Float.MIN_VALUE) + p_float(Float.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + p_float(0.0)))
  }

  function testFloatFloatAddition() {
    assertEquals(b_float(0.0), b_float(0.0) + b_float(0.0))
    assertEquals(b_float(23.0), b_float(0.0) + b_float(23.0))
    assertEquals(b_float(23.123), b_float(0.0) + b_float(23.123))
    assertEquals(b_float(-32.0), b_float(0.0) + b_float(-32.0))
    assertEquals(b_float(-32.456), b_float(0.0) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(0.0) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(0.0) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(0.0) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(0.0) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(0.0) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(23.0), b_float(23.0) + b_float(0.0))
    assertEquals(b_float(46.0), b_float(23.0) + b_float(23.0))
    assertEquals(b_float(46.123), b_float(23.0) + b_float(23.123))
    assertEquals(b_float(-9.0), b_float(23.0) + b_float(-32.0))
    assertEquals(b_float(-9.456001), b_float(23.0) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(23.0) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(23.0) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(23.0) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(23.0) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(23.0), b_float(23.0) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(23.123), b_float(23.123) + b_float(0.0))
    assertEquals(b_float(46.123), b_float(23.123) + b_float(23.0))
    assertEquals(b_float(46.246), b_float(23.123) + b_float(23.123))
    assertEquals(b_float(-8.877001), b_float(23.123) + b_float(-32.0))
    assertEquals(b_float(-9.333002), b_float(23.123) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(23.123) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(23.123) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(23.123) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(23.123) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(23.123), b_float(23.123) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(-32.0), b_float(-32.0) + b_float(0.0))
    assertEquals(b_float(-9.0), b_float(-32.0) + b_float(23.0))
    assertEquals(b_float(-8.877001), b_float(-32.0) + b_float(23.123))
    assertEquals(b_float(-64.0), b_float(-32.0) + b_float(-32.0))
    assertEquals(b_float(-64.456), b_float(-32.0) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(-32.0) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(-32.0) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(-32.0) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(-32.0) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(-32.0), b_float(-32.0) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(-32.456), b_float(-32.456) + b_float(0.0))
    assertEquals(b_float(-9.456001), b_float(-32.456) + b_float(23.0))
    assertEquals(b_float(-9.333002), b_float(-32.456) + b_float(23.123))
    assertEquals(b_float(-64.456), b_float(-32.456) + b_float(-32.0))
    assertEquals(b_float(-64.912), b_float(-32.456) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(-32.456) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(-32.456) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(-32.456) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(-32.456) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(-32.456), b_float(-32.456) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(0.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(23.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(23.123))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(-32.0))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.POSITIVE_INFINITY) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.POSITIVE_INFINITY) + b_float(Float.NaN_))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(0.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(23.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(23.123))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(-32.0))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(-32.456))
    assertEquals(b_float(Float.NaN_), b_float(Float.NEGATIVE_INFINITY) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NEGATIVE_INFINITY) + b_float(Float.NaN_))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(0.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(23.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(23.123))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(-32.0))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(-32.456))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(Float.NaN_))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.NaN_), b_float(Float.NaN_) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(0.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(23.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(23.123))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(-32.0))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.MAX_VALUE) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.MAX_VALUE) + b_float(Float.NaN_))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MAX_VALUE) + b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.parseFloat("1.4E-45")), b_float(Float.MIN_VALUE) + b_float(0.0))
    assertEquals(b_float(23.0), b_float(Float.MIN_VALUE) + b_float(23.0))
    assertEquals(b_float(23.123), b_float(Float.MIN_VALUE) + b_float(23.123))
    assertEquals(b_float(-32.0), b_float(Float.MIN_VALUE) + b_float(-32.0))
    assertEquals(b_float(-32.456), b_float(Float.MIN_VALUE) + b_float(-32.456))
    assertEquals(b_float(Float.POSITIVE_INFINITY), b_float(Float.MIN_VALUE) + b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.NEGATIVE_INFINITY), b_float(Float.MIN_VALUE) + b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_float(Float.MIN_VALUE) + b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("3.4028235E38")), b_float(Float.MIN_VALUE) + b_float(Float.MAX_VALUE))
    assertEquals(b_float(Float.parseFloat("2.8E-45")), b_float(Float.MIN_VALUE) + b_float(Float.MIN_VALUE))

    assertEquals(Float, statictypeof(b_float(0.0) + b_float(0.0)))
  }

  function testFloatPDoubleAddition() {
    assertEquals(b_double(0.0), b_float(0.0) + p_double(0.0))
    assertEquals(b_double(23.0), b_float(0.0) + p_double(23.0))
    assertEquals(b_double(23.123), b_float(0.0) + p_double(23.123))
    assertEquals(b_double(-32.0), b_float(0.0) + p_double(-32.0))
    assertEquals(b_double(-32.456), b_float(0.0) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(0.0) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(0.0) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(0.0) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(0.0) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.MIN_VALUE), b_float(0.0) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(23.0), b_float(23.0) + p_double(0.0))
    assertEquals(b_double(46.0), b_float(23.0) + p_double(23.0))
    assertEquals(b_double(46.123000000000005), b_float(23.0) + p_double(23.123))
    assertEquals(b_double(-9.0), b_float(23.0) + p_double(-32.0))
    assertEquals(b_double(-9.456000000000003), b_float(23.0) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(23.0) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(23.0) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(23.0) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(23.0) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(23.0), b_float(23.0) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(23.12299919128418), b_float(23.123) + p_double(0.0))
    assertEquals(b_double(46.12299919128418), b_float(23.123) + p_double(23.0))
    assertEquals(b_double(46.245999191284184), b_float(23.123) + p_double(23.123))
    assertEquals(b_double(-8.87700080871582), b_float(23.123) + p_double(-32.0))
    assertEquals(b_double(-9.333000808715823), b_float(23.123) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(23.123) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(23.123) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(23.123) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(23.123) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(23.12299919128418), b_float(23.123) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(-32.0), b_float(-32.0) + p_double(0.0))
    assertEquals(b_double(-9.0), b_float(-32.0) + p_double(23.0))
    assertEquals(b_double(-8.876999999999999), b_float(-32.0) + p_double(23.123))
    assertEquals(b_double(-64.0), b_float(-32.0) + p_double(-32.0))
    assertEquals(b_double(-64.456), b_float(-32.0) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(-32.0) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(-32.0) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(-32.0) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(-32.0) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(-32.0), b_float(-32.0) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(-32.45600128173828), b_float(-32.456) + p_double(0.0))
    assertEquals(b_double(-9.456001281738281), b_float(-32.456) + p_double(23.0))
    assertEquals(b_double(-9.33300128173828), b_float(-32.456) + p_double(23.123))
    assertEquals(b_double(-64.45600128173828), b_float(-32.456) + p_double(-32.0))
    assertEquals(b_double(-64.91200128173828), b_float(-32.456) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(-32.456) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(-32.456) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(-32.456) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(-32.456) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(-32.45600128173828), b_float(-32.456) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(0.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(23.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(23.123))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(-32.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.POSITIVE_INFINITY) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.POSITIVE_INFINITY) + p_double(Double.NaN_))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(0.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(23.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(23.123))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(-32.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(-32.456))
    assertEquals(b_double(Double.NaN_), b_float(Float.NEGATIVE_INFINITY) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NEGATIVE_INFINITY) + p_double(Double.NaN_))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(0.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(23.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(23.123))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(-32.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(-32.456))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(Double.NaN_))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(0.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(23.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(23.123))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(-32.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.MAX_VALUE) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.MAX_VALUE) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(Float.MAX_VALUE) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.parseDouble("1.401298464324817E-45")), b_float(Float.MIN_VALUE) + p_double(0.0))
    assertEquals(b_double(23.0), b_float(Float.MIN_VALUE) + p_double(23.0))
    assertEquals(b_double(23.123), b_float(Float.MIN_VALUE) + p_double(23.123))
    assertEquals(b_double(-32.0), b_float(Float.MIN_VALUE) + p_double(-32.0))
    assertEquals(b_double(-32.456), b_float(Float.MIN_VALUE) + p_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.MIN_VALUE) + p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.MIN_VALUE) + p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.MIN_VALUE) + p_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(Float.MIN_VALUE) + p_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.parseDouble("1.401298464324817E-45")), b_float(Float.MIN_VALUE) + p_double(Double.MIN_VALUE))

    assertEquals(Double, statictypeof(b_float(0.0) + p_double(0.0)))
  }

  function testFloatDoubleAddition() {
    assertEquals(b_double(0.0), b_float(0.0) + b_double(0.0))
    assertEquals(b_double(23.0), b_float(0.0) + b_double(23.0))
    assertEquals(b_double(23.123), b_float(0.0) + b_double(23.123))
    assertEquals(b_double(-32.0), b_float(0.0) + b_double(-32.0))
    assertEquals(b_double(-32.456), b_float(0.0) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(0.0) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(0.0) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(0.0) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(0.0) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.MIN_VALUE), b_float(0.0) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(23.0), b_float(23.0) + b_double(0.0))
    assertEquals(b_double(46.0), b_float(23.0) + b_double(23.0))
    assertEquals(b_double(46.123000000000005), b_float(23.0) + b_double(23.123))
    assertEquals(b_double(-9.0), b_float(23.0) + b_double(-32.0))
    assertEquals(b_double(-9.456000000000003), b_float(23.0) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(23.0) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(23.0) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(23.0) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(23.0) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(23.0), b_float(23.0) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(23.12299919128418), b_float(23.123) + b_double(0.0))
    assertEquals(b_double(46.12299919128418), b_float(23.123) + b_double(23.0))
    assertEquals(b_double(46.245999191284184), b_float(23.123) + b_double(23.123))
    assertEquals(b_double(-8.87700080871582), b_float(23.123) + b_double(-32.0))
    assertEquals(b_double(-9.333000808715823), b_float(23.123) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(23.123) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(23.123) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(23.123) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(23.123) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(23.12299919128418), b_float(23.123) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(-32.0), b_float(-32.0) + b_double(0.0))
    assertEquals(b_double(-9.0), b_float(-32.0) + b_double(23.0))
    assertEquals(b_double(-8.876999999999999), b_float(-32.0) + b_double(23.123))
    assertEquals(b_double(-64.0), b_float(-32.0) + b_double(-32.0))
    assertEquals(b_double(-64.456), b_float(-32.0) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(-32.0) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(-32.0) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(-32.0) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(-32.0) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(-32.0), b_float(-32.0) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(-32.45600128173828), b_float(-32.456) + b_double(0.0))
    assertEquals(b_double(-9.456001281738281), b_float(-32.456) + b_double(23.0))
    assertEquals(b_double(-9.33300128173828), b_float(-32.456) + b_double(23.123))
    assertEquals(b_double(-64.45600128173828), b_float(-32.456) + b_double(-32.0))
    assertEquals(b_double(-64.91200128173828), b_float(-32.456) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(-32.456) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(-32.456) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(-32.456) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(-32.456) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(-32.45600128173828), b_float(-32.456) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(0.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(23.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(23.123))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(-32.0))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.POSITIVE_INFINITY) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.POSITIVE_INFINITY) + b_double(Double.NaN_))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.POSITIVE_INFINITY) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(0.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(23.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(23.123))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(-32.0))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(-32.456))
    assertEquals(b_double(Double.NaN_), b_float(Float.NEGATIVE_INFINITY) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NEGATIVE_INFINITY) + b_double(Double.NaN_))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.NEGATIVE_INFINITY) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(0.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(23.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(23.123))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(-32.0))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(-32.456))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(Double.NaN_))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.NaN_), b_float(Float.NaN_) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(0.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(23.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(23.123))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(-32.0))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.MAX_VALUE) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.MAX_VALUE) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.MAX_VALUE) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(Float.MAX_VALUE) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.parseDouble("3.4028234663852886E38")), b_float(Float.MAX_VALUE) + b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.parseDouble("1.401298464324817E-45")), b_float(Float.MIN_VALUE) + b_double(0.0))
    assertEquals(b_double(23.0), b_float(Float.MIN_VALUE) + b_double(23.0))
    assertEquals(b_double(23.123), b_float(Float.MIN_VALUE) + b_double(23.123))
    assertEquals(b_double(-32.0), b_float(Float.MIN_VALUE) + b_double(-32.0))
    assertEquals(b_double(-32.456), b_float(Float.MIN_VALUE) + b_double(-32.456))
    assertEquals(b_double(Double.POSITIVE_INFINITY), b_float(Float.MIN_VALUE) + b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.NEGATIVE_INFINITY), b_float(Float.MIN_VALUE) + b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_float(Float.MIN_VALUE) + b_double(Double.NaN_))
    assertEquals(b_double(Double.MAX_VALUE), b_float(Float.MIN_VALUE) + b_double(Double.MAX_VALUE))
    assertEquals(b_double(Double.parseDouble("1.401298464324817E-45")), b_float(Float.MIN_VALUE) + b_double(Double.MIN_VALUE))

    assertEquals(Double, statictypeof(b_float(0.0) + b_double(0.0)))
  }

  function testFloatBigIntegerAddition() {
    assertEquals(big_decimal("0.0"), b_float(0.0) + big_int("0"))
    assertEquals(big_decimal("23.0"), b_float(0.0) + big_int("23"))
    assertEquals(big_decimal("-32.0"), b_float(0.0) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567890.0"), b_float(0.0) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567890.0"), b_float(0.0) + big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("23.0"), b_float(23.0) + big_int("0"))
    assertEquals(big_decimal("46.0"), b_float(23.0) + big_int("23"))
    assertEquals(big_decimal("-9.0"), b_float(23.0) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567913.0"), b_float(23.0) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567867.0"), b_float(23.0) + big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("23.123"), b_float(23.123) + big_int("0"))
    assertEquals(big_decimal("46.123"), b_float(23.123) + big_int("23"))
    assertEquals(big_decimal("-8.877"), b_float(23.123) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567913.123"), b_float(23.123) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567866.877"), b_float(23.123) + big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("-32.0"), b_float(-32.0) + big_int("0"))
    assertEquals(big_decimal("-9.0"), b_float(-32.0) + big_int("23"))
    assertEquals(big_decimal("-64.0"), b_float(-32.0) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567858.0"), b_float(-32.0) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567922.0"), b_float(-32.0) + big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("-32.456"), b_float(-32.456) + big_int("0"))
    assertEquals(big_decimal("-9.456"), b_float(-32.456) + big_int("23"))
    assertEquals(big_decimal("-64.456"), b_float(-32.456) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567857.544"), b_float(-32.456) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567922.456"), b_float(-32.456) + big_int("-123456789012345678901234567890"))

    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_int("0"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_int("23"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_int("-32"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_int("123456789012345678901234567890"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_int("-123456789012345678901234567890"))

    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_int("0"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_int("23"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_int("-32"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_int("123456789012345678901234567890"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_int("-123456789012345678901234567890"))

    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_int("0"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_int("23"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_int("-32"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_int("123456789012345678901234567890"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("340282350000000000000000000000000000000"), b_float(Float.MAX_VALUE) + big_int("0"))
    assertEquals(big_decimal("340282350000000000000000000000000000023"), b_float(Float.MAX_VALUE) + big_int("23"))
    assertEquals(big_decimal("340282349999999999999999999999999999968"), b_float(Float.MAX_VALUE) + big_int("-32"))
    assertEquals(big_decimal("340282350123456789012345678901234567890"), b_float(Float.MAX_VALUE) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("340282349876543210987654321098765432110"), b_float(Float.MAX_VALUE) + big_int("-123456789012345678901234567890"))

    assertEquals(big_decimal("1.4E-45"), b_float(Float.MIN_VALUE) + big_int("0"))
    assertEquals(big_decimal("23.0000000000000000000000000000000000000000000014"), b_float(Float.MIN_VALUE) + big_int("23"))
    assertEquals(big_decimal("-31.9999999999999999999999999999999999999999999986"), b_float(Float.MIN_VALUE) + big_int("-32"))
    assertEquals(big_decimal("123456789012345678901234567890.0000000000000000000000000000000000000000000014"), b_float(Float.MIN_VALUE) + big_int("123456789012345678901234567890"))
    assertEquals(big_decimal("-123456789012345678901234567889.9999999999999999999999999999999999999999999986"), b_float(Float.MIN_VALUE) + big_int("-123456789012345678901234567890"))

    assertEquals(BigDecimal, statictypeof(b_float(0.0) + big_int("0")))
  }

  function testFloatBigDecimalAddition() {
    assertEquals(big_decimal("0.0"), b_float(0.0) + big_decimal("0"))
    assertEquals(big_decimal("23.0"), b_float(0.0) + big_decimal("23"))
    assertEquals(big_decimal("23.123"), b_float(0.0) + big_decimal("23.123"))
    assertEquals(big_decimal("-32.0"), b_float(0.0) + big_decimal("-32"))
    assertEquals(big_decimal("-32.456"), b_float(0.0) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567890.123456789"), b_float(0.0) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567890.123456789"), b_float(0.0) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("23.0"), b_float(23.0) + big_decimal("0"))
    assertEquals(big_decimal("46.0"), b_float(23.0) + big_decimal("23"))
    assertEquals(big_decimal("46.123"), b_float(23.0) + big_decimal("23.123"))
    assertEquals(big_decimal("-9.0"), b_float(23.0) + big_decimal("-32"))
    assertEquals(big_decimal("-9.456"), b_float(23.0) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567913.123456789"), b_float(23.0) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567867.123456789"), b_float(23.0) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("23.123"), b_float(23.123) + big_decimal("0"))
    assertEquals(big_decimal("46.123"), b_float(23.123) + big_decimal("23"))
    assertEquals(big_decimal("46.246"), b_float(23.123) + big_decimal("23.123"))
    assertEquals(big_decimal("-8.877"), b_float(23.123) + big_decimal("-32"))
    assertEquals(big_decimal("-9.333"), b_float(23.123) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567913.246456789"), b_float(23.123) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567867.000456789"), b_float(23.123) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("-32.0"), b_float(-32.0) + big_decimal("0"))
    assertEquals(big_decimal("-9.0"), b_float(-32.0) + big_decimal("23"))
    assertEquals(big_decimal("-8.877"), b_float(-32.0) + big_decimal("23.123"))
    assertEquals(big_decimal("-64.0"), b_float(-32.0) + big_decimal("-32"))
    assertEquals(big_decimal("-64.456"), b_float(-32.0) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567858.123456789"), b_float(-32.0) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567922.123456789"), b_float(-32.0) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("-32.456"), b_float(-32.456) + big_decimal("0"))
    assertEquals(big_decimal("-9.456"), b_float(-32.456) + big_decimal("23"))
    assertEquals(big_decimal("-9.333"), b_float(-32.456) + big_decimal("23.123"))
    assertEquals(big_decimal("-64.456"), b_float(-32.456) + big_decimal("-32"))
    assertEquals(big_decimal("-64.912"), b_float(-32.456) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567857.667456789"), b_float(-32.456) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567922.579456789"), b_float(-32.456) + big_decimal("-123456789012345678901234567890.123456789"))

    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("0"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("23"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("23.123"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("-32"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("-32.456"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("123456789012345678901234567890.123456789"))
    // Skipped test assertEquals(something, b_float(Float.POSITIVE_INFINITY) - big_decimal("-123456789012345678901234567890.123456789"))

    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("0"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("23"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("23.123"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("-32"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("-32.456"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("123456789012345678901234567890.123456789"))
    // Skipped test assertEquals(something, b_float(Float.NEGATIVE_INFINITY) - big_decimal("-123456789012345678901234567890.123456789"))

    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("0"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("23"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("23.123"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("-32"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("-32.456"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("123456789012345678901234567890.123456789"))
    // Skipped test assertEquals(something, b_float(Float.NaN_) - big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("340282350000000000000000000000000000000"), b_float(Float.MAX_VALUE) + big_decimal("0"))
    assertEquals(big_decimal("340282350000000000000000000000000000023"), b_float(Float.MAX_VALUE) + big_decimal("23"))
    assertEquals(big_decimal("340282350000000000000000000000000000023.123"), b_float(Float.MAX_VALUE) + big_decimal("23.123"))
    assertEquals(big_decimal("340282349999999999999999999999999999968"), b_float(Float.MAX_VALUE) + big_decimal("-32"))
    assertEquals(big_decimal("340282349999999999999999999999999999967.544"), b_float(Float.MAX_VALUE) + big_decimal("-32.456"))
    assertEquals(big_decimal("340282350123456789012345678901234567890.123456789"), b_float(Float.MAX_VALUE) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("340282349876543210987654321098765432109.876543211"), b_float(Float.MAX_VALUE) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(big_decimal("1.4E-45"), b_float(Float.MIN_VALUE) + big_decimal("0"))
    assertEquals(big_decimal("23.0000000000000000000000000000000000000000000014"), b_float(Float.MIN_VALUE) + big_decimal("23"))
    assertEquals(big_decimal("23.1230000000000000000000000000000000000000000014"), b_float(Float.MIN_VALUE) + big_decimal("23.123"))
    assertEquals(big_decimal("-31.9999999999999999999999999999999999999999999986"), b_float(Float.MIN_VALUE) + big_decimal("-32"))
    assertEquals(big_decimal("-32.4559999999999999999999999999999999999999999986"), b_float(Float.MIN_VALUE) + big_decimal("-32.456"))
    assertEquals(big_decimal("123456789012345678901234567890.1234567890000000000000000000000000000000000014"), b_float(Float.MIN_VALUE) + big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("-123456789012345678901234567890.1234567889999999999999999999999999999999999986"), b_float(Float.MIN_VALUE) + big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(BigDecimal, statictypeof(b_float(0.0) + big_decimal("0")))
  }

}

