package gw.spec.core.expressions.arithmetic.remainder
uses java.lang.Byte
uses java.lang.Character
uses java.lang.Integer
uses java.lang.Short
uses java.lang.Long
uses java.lang.Float
uses java.lang.Double
uses java.math.BigInteger
uses java.math.BigDecimal
uses gw.spec.core.expressions.arithmetic.ArithmeticTestBase

class Remainder_ByteTest extends ArithmeticTestBase {

  function testBytePByteRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % p_byte(0))
    assertEquals(b_int(0), b_byte(0) % p_byte(1))
    assertEquals(b_int(0), b_byte(0) % p_byte(-1))
    assertEquals(b_int(0), b_byte(0) % p_byte(23))
    assertEquals(b_int(0), b_byte(0) % p_byte(-32))
    assertEquals(b_int(0), b_byte(0) % p_byte(Byte.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % p_byte(0))
    assertEquals(p_int(0), p_byte(1) % p_byte(1))
    assertEquals(p_int(0), p_byte(1) % p_byte(-1))
    assertEquals(p_int(1), p_byte(1) % p_byte(23))
    assertEquals(p_int(1), p_byte(1) % p_byte(-32))
    assertEquals(p_int(1), p_byte(1) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_int(1), p_byte(1) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % p_byte(0))
    assertEquals(p_int(0), p_byte(-1) % p_byte(1))
    assertEquals(p_int(0), p_byte(-1) % p_byte(-1))
    assertEquals(p_int(-1), p_byte(-1) % p_byte(23))
    assertEquals(p_int(-1), p_byte(-1) % p_byte(-32))
    assertEquals(p_int(-1), p_byte(-1) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_int(-1), p_byte(-1) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % p_byte(0))
    assertEquals(b_int(0), b_byte(23) % p_byte(1))
    assertEquals(b_int(0), b_byte(23) % p_byte(-1))
    assertEquals(b_int(0), b_byte(23) % p_byte(23))
    assertEquals(b_int(23), b_byte(23) % p_byte(-32))
    assertEquals(b_int(23), b_byte(23) % p_byte(Byte.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % p_byte(0))
    assertEquals(b_int(0), b_byte(-32) % p_byte(1))
    assertEquals(b_int(0), b_byte(-32) % p_byte(-1))
    assertEquals(b_int(-9), b_byte(-32) % p_byte(23))
    assertEquals(b_int(0), b_byte(-32) % p_byte(-32))
    assertEquals(b_int(-32), b_byte(-32) % p_byte(Byte.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % p_byte(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_byte(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_byte(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % p_byte(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % p_byte(-32))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_byte(Byte.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % p_byte(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % p_byte(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(-32))
    assertEquals(b_int(-1), b_byte(Byte.MIN_VALUE) % p_byte(Byte.MAX_VALUE))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(Byte.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % p_byte(0)))
  }

  function testByteByteRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % b_byte(0))
    assertEquals(b_int(0), b_byte(0) % p_byte(1))
    assertEquals(b_int(0), b_byte(0) % p_byte(-1))
    assertEquals(b_int(0), b_byte(0) % b_byte(23))
    assertEquals(b_int(0), b_byte(0) % b_byte(-32))
    assertEquals(b_int(0), b_byte(0) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % b_byte(0))
    assertEquals(p_int(0), p_byte(1) % p_byte(1))
    assertEquals(p_int(0), p_byte(1) % p_byte(-1))
    assertEquals(b_int(1), p_byte(1) % b_byte(23))
    assertEquals(b_int(1), p_byte(1) % b_byte(-32))
    assertEquals(b_int(1), p_byte(1) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(1), p_byte(1) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % b_byte(0))
    assertEquals(p_int(0), p_byte(-1) % p_byte(1))
    assertEquals(p_int(0), p_byte(-1) % p_byte(-1))
    assertEquals(b_int(-1), p_byte(-1) % b_byte(23))
    assertEquals(b_int(-1), p_byte(-1) % b_byte(-32))
    assertEquals(b_int(-1), p_byte(-1) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(-1), p_byte(-1) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % b_byte(0))
    assertEquals(b_int(0), b_byte(23) % p_byte(1))
    assertEquals(b_int(0), b_byte(23) % p_byte(-1))
    assertEquals(b_int(0), b_byte(23) % b_byte(23))
    assertEquals(b_int(23), b_byte(23) % b_byte(-32))
    assertEquals(b_int(23), b_byte(23) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % b_byte(0))
    assertEquals(b_int(0), b_byte(-32) % p_byte(1))
    assertEquals(b_int(0), b_byte(-32) % p_byte(-1))
    assertEquals(b_int(-9), b_byte(-32) % b_byte(23))
    assertEquals(b_int(0), b_byte(-32) % b_byte(-32))
    assertEquals(b_int(-32), b_byte(-32) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % b_byte(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_byte(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_byte(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % b_byte(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % b_byte(-32))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % b_byte(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_byte(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % b_byte(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_byte(-32))
    assertEquals(b_int(-1), b_byte(Byte.MIN_VALUE) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_byte(Byte.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % b_byte(0)))
  }

  function testBytePShortRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % p_short(0))
    assertEquals(b_int(0), b_byte(0) % p_short(1))
    assertEquals(b_int(0), b_byte(0) % p_short(-1))
    assertEquals(b_int(0), b_byte(0) % p_short(23))
    assertEquals(b_int(0), b_byte(0) % p_short(-32))
    assertEquals(b_int(0), b_byte(0) % p_short(Short.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % p_short(0))
    assertEquals(p_int(0), p_byte(1) % p_short(1))
    assertEquals(p_int(0), p_byte(1) % p_short(-1))
    assertEquals(p_int(1), p_byte(1) % p_short(23))
    assertEquals(p_int(1), p_byte(1) % p_short(-32))
    assertEquals(p_int(1), p_byte(1) % p_short(Short.MAX_VALUE))
    assertEquals(p_int(1), p_byte(1) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % p_short(0))
    assertEquals(p_int(0), p_byte(-1) % p_short(1))
    assertEquals(p_int(0), p_byte(-1) % p_short(-1))
    assertEquals(p_int(-1), p_byte(-1) % p_short(23))
    assertEquals(p_int(-1), p_byte(-1) % p_short(-32))
    assertEquals(p_int(-1), p_byte(-1) % p_short(Short.MAX_VALUE))
    assertEquals(p_int(-1), p_byte(-1) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % p_short(0))
    assertEquals(b_int(0), b_byte(23) % p_short(1))
    assertEquals(b_int(0), b_byte(23) % p_short(-1))
    assertEquals(b_int(0), b_byte(23) % p_short(23))
    assertEquals(b_int(23), b_byte(23) % p_short(-32))
    assertEquals(b_int(23), b_byte(23) % p_short(Short.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % p_short(0))
    assertEquals(b_int(0), b_byte(-32) % p_short(1))
    assertEquals(b_int(0), b_byte(-32) % p_short(-1))
    assertEquals(b_int(-9), b_byte(-32) % p_short(23))
    assertEquals(b_int(0), b_byte(-32) % p_short(-32))
    assertEquals(b_int(-32), b_byte(-32) % p_short(Short.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % p_short(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_short(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_short(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % p_short(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % p_short(-32))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_short(Short.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % p_short(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_short(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_short(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % p_short(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_short(-32))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % p_short(Short.MAX_VALUE))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % p_short(Short.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % p_short(0)))
  }

  function testByteShortRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % b_short(0))
    assertEquals(b_int(0), b_byte(0) % b_short(1))
    assertEquals(b_int(0), b_byte(0) % b_short(-1))
    assertEquals(b_int(0), b_byte(0) % b_short(23))
    assertEquals(b_int(0), b_byte(0) % b_short(-32))
    assertEquals(b_int(0), b_byte(0) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % b_short(0))
    assertEquals(b_int(0), p_byte(1) % b_short(1))
    assertEquals(b_int(0), p_byte(1) % b_short(-1))
    assertEquals(b_int(1), p_byte(1) % b_short(23))
    assertEquals(b_int(1), p_byte(1) % b_short(-32))
    assertEquals(b_int(1), p_byte(1) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(1), p_byte(1) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % b_short(0))
    assertEquals(b_int(0), p_byte(-1) % b_short(1))
    assertEquals(b_int(0), p_byte(-1) % b_short(-1))
    assertEquals(b_int(-1), p_byte(-1) % b_short(23))
    assertEquals(b_int(-1), p_byte(-1) % b_short(-32))
    assertEquals(b_int(-1), p_byte(-1) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(-1), p_byte(-1) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % b_short(0))
    assertEquals(b_int(0), b_byte(23) % b_short(1))
    assertEquals(b_int(0), b_byte(23) % b_short(-1))
    assertEquals(b_int(0), b_byte(23) % b_short(23))
    assertEquals(b_int(23), b_byte(23) % b_short(-32))
    assertEquals(b_int(23), b_byte(23) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % b_short(0))
    assertEquals(b_int(0), b_byte(-32) % b_short(1))
    assertEquals(b_int(0), b_byte(-32) % b_short(-1))
    assertEquals(b_int(-9), b_byte(-32) % b_short(23))
    assertEquals(b_int(0), b_byte(-32) % b_short(-32))
    assertEquals(b_int(-32), b_byte(-32) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % b_short(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_short(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_short(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % b_short(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % b_short(-32))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % b_short(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_short(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_short(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % b_short(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_short(-32))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % b_short(Short.MAX_VALUE))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % b_short(Short.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % b_short(0)))
  }

  function testBytePCharRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % p_char(0))
    assertEquals(b_int(0), b_byte(0) % p_char(1))
    assertEquals(b_int(0), b_byte(0) % p_char(23))
    assertEquals(b_int(0), b_byte(0) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % p_char(0))
    assertEquals(p_int(0), p_byte(1) % p_char(1))
    assertEquals(p_int(1), p_byte(1) % p_char(23))
    assertEquals(p_int(1), p_byte(1) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % p_char(0))
    assertEquals(p_int(0), p_byte(-1) % p_char(1))
    assertEquals(p_int(-1), p_byte(-1) % p_char(23))
    assertEquals(p_int(-1), p_byte(-1) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % p_char(0))
    assertEquals(b_int(0), b_byte(23) % p_char(1))
    assertEquals(b_int(0), b_byte(23) % p_char(23))
    assertEquals(b_int(23), b_byte(23) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % p_char(0))
    assertEquals(b_int(0), b_byte(-32) % p_char(1))
    assertEquals(b_int(-9), b_byte(-32) % p_char(23))
    assertEquals(b_int(-32), b_byte(-32) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % p_char(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_char(1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % p_char(23))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % p_char(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_char(1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % p_char(23))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % p_char(Character.MAX_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % p_char(0)))
  }

  function testByteCharacterRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % b_char(0))
    assertEquals(b_int(0), b_byte(0) % b_char(1))
    assertEquals(b_int(0), b_byte(0) % b_char(23))
    assertEquals(b_int(0), b_byte(0) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % b_char(0))
    assertEquals(b_int(0), p_byte(1) % b_char(1))
    assertEquals(b_int(1), p_byte(1) % b_char(23))
    assertEquals(b_int(1), p_byte(1) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % b_char(0))
    assertEquals(b_int(0), p_byte(-1) % b_char(1))
    assertEquals(b_int(-1), p_byte(-1) % b_char(23))
    assertEquals(b_int(-1), p_byte(-1) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % b_char(0))
    assertEquals(b_int(0), b_byte(23) % b_char(1))
    assertEquals(b_int(0), b_byte(23) % b_char(23))
    assertEquals(b_int(23), b_byte(23) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % b_char(0))
    assertEquals(b_int(0), b_byte(-32) % b_char(1))
    assertEquals(b_int(-9), b_byte(-32) % b_char(23))
    assertEquals(b_int(-32), b_byte(-32) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % b_char(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_char(1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % b_char(23))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % b_char(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_char(1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % b_char(23))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % b_char(Character.MAX_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % b_char(0)))
  }

  function testBytePIntRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % p_int(0))
    assertEquals(b_int(0), b_byte(0) % p_int(1))
    assertEquals(b_int(0), b_byte(0) % p_int(-1))
    assertEquals(b_int(0), b_byte(0) % p_int(23))
    assertEquals(b_int(0), b_byte(0) % p_int(-32))
    assertEquals(b_int(0), b_byte(0) % p_int(Integer.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % p_int(0))
    assertEquals(p_int(0), p_byte(1) % p_int(1))
    assertEquals(p_int(0), p_byte(1) % p_int(-1))
    assertEquals(p_int(1), p_byte(1) % p_int(23))
    assertEquals(p_int(1), p_byte(1) % p_int(-32))
    assertEquals(p_int(1), p_byte(1) % p_int(Integer.MAX_VALUE))
    assertEquals(p_int(1), p_byte(1) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % p_int(0))
    assertEquals(p_int(0), p_byte(-1) % p_int(1))
    assertEquals(p_int(0), p_byte(-1) % p_int(-1))
    assertEquals(p_int(-1), p_byte(-1) % p_int(23))
    assertEquals(p_int(-1), p_byte(-1) % p_int(-32))
    assertEquals(p_int(-1), p_byte(-1) % p_int(Integer.MAX_VALUE))
    assertEquals(p_int(-1), p_byte(-1) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % p_int(0))
    assertEquals(b_int(0), b_byte(23) % p_int(1))
    assertEquals(b_int(0), b_byte(23) % p_int(-1))
    assertEquals(b_int(0), b_byte(23) % p_int(23))
    assertEquals(b_int(23), b_byte(23) % p_int(-32))
    assertEquals(b_int(23), b_byte(23) % p_int(Integer.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % p_int(0))
    assertEquals(b_int(0), b_byte(-32) % p_int(1))
    assertEquals(b_int(0), b_byte(-32) % p_int(-1))
    assertEquals(b_int(-9), b_byte(-32) % p_int(23))
    assertEquals(b_int(0), b_byte(-32) % p_int(-32))
    assertEquals(b_int(-32), b_byte(-32) % p_int(Integer.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % p_int(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_int(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % p_int(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % p_int(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % p_int(-32))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_int(Integer.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % p_int(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_int(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_int(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % p_int(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % p_int(-32))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % p_int(Integer.MAX_VALUE))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % p_int(Integer.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % p_int(0)))
  }

  function testByteIntegerRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % b_int(0))
    assertEquals(b_int(0), b_byte(0) % b_int(1))
    assertEquals(b_int(0), b_byte(0) % b_int(-1))
    assertEquals(b_int(0), b_byte(0) % b_int(23))
    assertEquals(b_int(0), b_byte(0) % b_int(-32))
    assertEquals(b_int(0), b_byte(0) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(0), b_byte(0) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % b_int(0))
    assertEquals(b_int(0), p_byte(1) % b_int(1))
    assertEquals(b_int(0), p_byte(1) % b_int(-1))
    assertEquals(b_int(1), p_byte(1) % b_int(23))
    assertEquals(b_int(1), p_byte(1) % b_int(-32))
    assertEquals(b_int(1), p_byte(1) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(1), p_byte(1) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % b_int(0))
    assertEquals(b_int(0), p_byte(-1) % b_int(1))
    assertEquals(b_int(0), p_byte(-1) % b_int(-1))
    assertEquals(b_int(-1), p_byte(-1) % b_int(23))
    assertEquals(b_int(-1), p_byte(-1) % b_int(-32))
    assertEquals(b_int(-1), p_byte(-1) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(-1), p_byte(-1) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % b_int(0))
    assertEquals(b_int(0), b_byte(23) % b_int(1))
    assertEquals(b_int(0), b_byte(23) % b_int(-1))
    assertEquals(b_int(0), b_byte(23) % b_int(23))
    assertEquals(b_int(23), b_byte(23) % b_int(-32))
    assertEquals(b_int(23), b_byte(23) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(23), b_byte(23) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % b_int(0))
    assertEquals(b_int(0), b_byte(-32) % b_int(1))
    assertEquals(b_int(0), b_byte(-32) % b_int(-1))
    assertEquals(b_int(-9), b_byte(-32) % b_int(23))
    assertEquals(b_int(0), b_byte(-32) % b_int(-32))
    assertEquals(b_int(-32), b_byte(-32) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(-32), b_byte(-32) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % b_int(0))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_int(1))
    assertEquals(b_int(0), b_byte(Byte.MAX_VALUE) % b_int(-1))
    assertEquals(b_int(12), b_byte(Byte.MAX_VALUE) % b_int(23))
    assertEquals(b_int(31), b_byte(Byte.MAX_VALUE) % b_int(-32))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(127), b_byte(Byte.MAX_VALUE) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % b_int(0))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_int(1))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_int(-1))
    assertEquals(b_int(-13), b_byte(Byte.MIN_VALUE) % b_int(23))
    assertEquals(b_int(0), b_byte(Byte.MIN_VALUE) % b_int(-32))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % b_int(Integer.MAX_VALUE))
    assertEquals(b_int(-128), b_byte(Byte.MIN_VALUE) % b_int(Integer.MIN_VALUE))

    assertEquals(Integer, statictypeof(b_byte(0) % b_int(0)))
  }

  function testBytePLongRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % p_long(0))
    assertEquals(b_long(0), b_byte(0) % p_long(1))
    assertEquals(b_long(0), b_byte(0) % p_long(-1))
    assertEquals(b_long(0), b_byte(0) % p_long(23))
    assertEquals(b_long(0), b_byte(0) % p_long(-32))
    assertEquals(b_long(0), b_byte(0) % p_long(Long.MAX_VALUE))
    assertEquals(b_long(0), b_byte(0) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % p_long(0))
    assertEquals(p_long(0), p_byte(1) % p_long(1))
    assertEquals(p_long(0), p_byte(1) % p_long(-1))
    assertEquals(p_long(1), p_byte(1) % p_long(23))
    assertEquals(p_long(1), p_byte(1) % p_long(-32))
    assertEquals(p_long(1), p_byte(1) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(1), p_byte(1) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % p_long(0))
    assertEquals(p_long(0), p_byte(-1) % p_long(1))
    assertEquals(p_long(0), p_byte(-1) % p_long(-1))
    assertEquals(p_long(-1), p_byte(-1) % p_long(23))
    assertEquals(p_long(-1), p_byte(-1) % p_long(-32))
    assertEquals(p_long(-1), p_byte(-1) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(-1), p_byte(-1) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % p_long(0))
    assertEquals(b_long(0), b_byte(23) % p_long(1))
    assertEquals(b_long(0), b_byte(23) % p_long(-1))
    assertEquals(b_long(0), b_byte(23) % p_long(23))
    assertEquals(b_long(23), b_byte(23) % p_long(-32))
    assertEquals(b_long(23), b_byte(23) % p_long(Long.MAX_VALUE))
    assertEquals(b_long(23), b_byte(23) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % p_long(0))
    assertEquals(b_long(0), b_byte(-32) % p_long(1))
    assertEquals(b_long(0), b_byte(-32) % p_long(-1))
    assertEquals(b_long(-9), b_byte(-32) % p_long(23))
    assertEquals(b_long(0), b_byte(-32) % p_long(-32))
    assertEquals(b_long(-32), b_byte(-32) % p_long(Long.MAX_VALUE))
    assertEquals(b_long(-32), b_byte(-32) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % p_long(0))
    assertEquals(b_long(0), b_byte(Byte.MAX_VALUE) % p_long(1))
    assertEquals(b_long(0), b_byte(Byte.MAX_VALUE) % p_long(-1))
    assertEquals(b_long(12), b_byte(Byte.MAX_VALUE) % p_long(23))
    assertEquals(b_long(31), b_byte(Byte.MAX_VALUE) % p_long(-32))
    assertEquals(b_long(127), b_byte(Byte.MAX_VALUE) % p_long(Long.MAX_VALUE))
    assertEquals(b_long(127), b_byte(Byte.MAX_VALUE) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % p_long(0))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % p_long(1))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % p_long(-1))
    assertEquals(b_long(-13), b_byte(Byte.MIN_VALUE) % p_long(23))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % p_long(-32))
    assertEquals(b_long(-128), b_byte(Byte.MIN_VALUE) % p_long(Long.MAX_VALUE))
    assertEquals(b_long(-128), b_byte(Byte.MIN_VALUE) % p_long(Long.MIN_VALUE))

    assertEquals(Long, statictypeof(b_byte(0) % p_long(0)))
  }

  function testByteLongRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> b_byte(0) % b_long(0))
    assertEquals(b_long(0), b_byte(0) % b_long(1))
    assertEquals(b_long(0), b_byte(0) % b_long(-1))
    assertEquals(b_long(0), b_byte(0) % b_long(23))
    assertEquals(b_long(0), b_byte(0) % b_long(-32))
    assertEquals(b_long(0), b_byte(0) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(0), b_byte(0) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(1) % b_long(0))
    assertEquals(b_long(0), p_byte(1) % b_long(1))
    assertEquals(b_long(0), p_byte(1) % b_long(-1))
    assertEquals(b_long(1), p_byte(1) % b_long(23))
    assertEquals(b_long(1), p_byte(1) % b_long(-32))
    assertEquals(b_long(1), p_byte(1) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(1), p_byte(1) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_byte(-1) % b_long(0))
    assertEquals(b_long(0), p_byte(-1) % b_long(1))
    assertEquals(b_long(0), p_byte(-1) % b_long(-1))
    assertEquals(b_long(-1), p_byte(-1) % b_long(23))
    assertEquals(b_long(-1), p_byte(-1) % b_long(-32))
    assertEquals(b_long(-1), p_byte(-1) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(-1), p_byte(-1) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(23) % b_long(0))
    assertEquals(b_long(0), b_byte(23) % b_long(1))
    assertEquals(b_long(0), b_byte(23) % b_long(-1))
    assertEquals(b_long(0), b_byte(23) % b_long(23))
    assertEquals(b_long(23), b_byte(23) % b_long(-32))
    assertEquals(b_long(23), b_byte(23) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(23), b_byte(23) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(-32) % b_long(0))
    assertEquals(b_long(0), b_byte(-32) % b_long(1))
    assertEquals(b_long(0), b_byte(-32) % b_long(-1))
    assertEquals(b_long(-9), b_byte(-32) % b_long(23))
    assertEquals(b_long(0), b_byte(-32) % b_long(-32))
    assertEquals(b_long(-32), b_byte(-32) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(-32), b_byte(-32) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MAX_VALUE) % b_long(0))
    assertEquals(b_long(0), b_byte(Byte.MAX_VALUE) % b_long(1))
    assertEquals(b_long(0), b_byte(Byte.MAX_VALUE) % b_long(-1))
    assertEquals(b_long(12), b_byte(Byte.MAX_VALUE) % b_long(23))
    assertEquals(b_long(31), b_byte(Byte.MAX_VALUE) % b_long(-32))
    assertEquals(b_long(127), b_byte(Byte.MAX_VALUE) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(127), b_byte(Byte.MAX_VALUE) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> b_byte(Byte.MIN_VALUE) % b_long(0))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % b_long(1))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % b_long(-1))
    assertEquals(b_long(-13), b_byte(Byte.MIN_VALUE) % b_long(23))
    assertEquals(b_long(0), b_byte(Byte.MIN_VALUE) % b_long(-32))
    assertEquals(b_long(-128), b_byte(Byte.MIN_VALUE) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(-128), b_byte(Byte.MIN_VALUE) % b_long(Long.MIN_VALUE))

    assertEquals(Long, statictypeof(b_byte(0) % b_long(0)))
  }

  function testBytePFloatRemainder() {
    assertEquals(b_float(Float.NaN_), b_byte(0) % p_float(0.0))
    assertEquals(b_float(0.0), b_byte(0) % p_float(1.0))
    assertEquals(b_float(0.0), b_byte(0) % p_float(-1.0))
    assertEquals(b_float(0.0), b_byte(0) % p_float(23.0))
    assertEquals(b_float(0.0), b_byte(0) % p_float(23.123))
    assertEquals(b_float(0.0), b_byte(0) % p_float(-32.0))
    assertEquals(b_float(0.0), b_byte(0) % p_float(-32.456))
    assertEquals(b_float(0.0), b_byte(0) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(0.0), b_byte(0) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(0) % p_float(Float.NaN_))
    assertEquals(b_float(0.0), b_byte(0) % p_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(0) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_byte(1) % p_float(0.0))
    assertEquals(p_float(0.0), p_byte(1) % p_float(1.0))
    assertEquals(p_float(0.0), p_byte(1) % p_float(-1.0))
    assertEquals(p_float(1.0), p_byte(1) % p_float(23.0))
    assertEquals(p_float(1.0), p_byte(1) % p_float(23.123))
    assertEquals(p_float(1.0), p_byte(1) % p_float(-32.0))
    assertEquals(p_float(1.0), p_byte(1) % p_float(-32.456))
    assertEquals(p_float(1.0), p_byte(1) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(1.0), p_byte(1) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_byte(1) % p_float(Float.NaN_))
    assertEquals(p_float(1.0), p_byte(1) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(0.0), p_byte(1) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_byte(-1) % p_float(0.0))
    assertEquals(p_float(-0.0), p_byte(-1) % p_float(1.0))
    assertEquals(p_float(-0.0), p_byte(-1) % p_float(-1.0))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(23.0))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(23.123))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(-32.0))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(-32.456))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_byte(-1) % p_float(Float.NaN_))
    assertEquals(p_float(-1.0), p_byte(-1) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(-0.0), p_byte(-1) % p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(23) % p_float(0.0))
    assertEquals(b_float(0.0), b_byte(23) % p_float(1.0))
    assertEquals(b_float(0.0), b_byte(23) % p_float(-1.0))
    assertEquals(b_float(0.0), b_byte(23) % p_float(23.0))
    assertEquals(b_float(23.0), b_byte(23) % p_float(23.123))
    assertEquals(b_float(23.0), b_byte(23) % p_float(-32.0))
    assertEquals(b_float(23.0), b_byte(23) % p_float(-32.456))
    assertEquals(b_float(23.0), b_byte(23) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(23.0), b_byte(23) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(23) % p_float(Float.NaN_))
    assertEquals(b_float(23.0), b_byte(23) % p_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(23) % p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(-32) % p_float(0.0))
    assertEquals(b_float(-0.0), b_byte(-32) % p_float(1.0))
    assertEquals(b_float(-0.0), b_byte(-32) % p_float(-1.0))
    assertEquals(b_float(-9.0), b_byte(-32) % p_float(23.0))
    assertEquals(b_float(-8.877001), b_byte(-32) % p_float(23.123))
    assertEquals(b_float(-0.0), b_byte(-32) % p_float(-32.0))
    assertEquals(b_float(-32.0), b_byte(-32) % p_float(-32.456))
    assertEquals(b_float(-32.0), b_byte(-32) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-32.0), b_byte(-32) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(-32) % p_float(Float.NaN_))
    assertEquals(b_float(-32.0), b_byte(-32) % p_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), b_byte(-32) % p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(Byte.MAX_VALUE) % p_float(0.0))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % p_float(1.0))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % p_float(-1.0))
    assertEquals(b_float(12.0), b_byte(Byte.MAX_VALUE) % p_float(23.0))
    assertEquals(b_float(11.385004), b_byte(Byte.MAX_VALUE) % p_float(23.123))
    assertEquals(b_float(31.0), b_byte(Byte.MAX_VALUE) % p_float(-32.0))
    assertEquals(b_float(29.631996), b_byte(Byte.MAX_VALUE) % p_float(-32.456))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(Byte.MAX_VALUE) % p_float(Float.NaN_))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % p_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % p_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(Byte.MIN_VALUE) % p_float(0.0))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % p_float(1.0))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % p_float(-1.0))
    assertEquals(b_float(-13.0), b_byte(Byte.MIN_VALUE) % p_float(23.0))
    assertEquals(b_float(-12.385004), b_byte(Byte.MIN_VALUE) % p_float(23.123))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % p_float(-32.0))
    assertEquals(b_float(-30.631996), b_byte(Byte.MIN_VALUE) % p_float(-32.456))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(Byte.MIN_VALUE) % p_float(Float.NaN_))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % p_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % p_float(Float.MIN_VALUE))

    assertEquals(Float, statictypeof(b_byte(0) % p_float(0.0)))
  }

  function testByteFloatRemainder() {
    assertEquals(b_float(Float.NaN_), b_byte(0) % b_float(0.0))
    assertEquals(b_float(0.0), b_byte(0) % b_float(1.0))
    assertEquals(b_float(0.0), b_byte(0) % b_float(-1.0))
    assertEquals(b_float(0.0), b_byte(0) % b_float(23.0))
    assertEquals(b_float(0.0), b_byte(0) % b_float(23.123))
    assertEquals(b_float(0.0), b_byte(0) % b_float(-32.0))
    assertEquals(b_float(0.0), b_byte(0) % b_float(-32.456))
    assertEquals(b_float(0.0), b_byte(0) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(0.0), b_byte(0) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(0) % b_float(Float.NaN_))
    assertEquals(b_float(0.0), b_byte(0) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(0) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_byte(1) % b_float(0.0))
    assertEquals(b_float(0.0), p_byte(1) % b_float(1.0))
    assertEquals(b_float(0.0), p_byte(1) % b_float(-1.0))
    assertEquals(b_float(1.0), p_byte(1) % b_float(23.0))
    assertEquals(b_float(1.0), p_byte(1) % b_float(23.123))
    assertEquals(b_float(1.0), p_byte(1) % b_float(-32.0))
    assertEquals(b_float(1.0), p_byte(1) % b_float(-32.456))
    assertEquals(b_float(1.0), p_byte(1) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(1.0), p_byte(1) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_byte(1) % b_float(Float.NaN_))
    assertEquals(b_float(1.0), p_byte(1) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), p_byte(1) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_byte(-1) % b_float(0.0))
    assertEquals(b_float(-0.0), p_byte(-1) % b_float(1.0))
    assertEquals(b_float(-0.0), p_byte(-1) % b_float(-1.0))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(23.0))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(23.123))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(-32.0))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(-32.456))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_byte(-1) % b_float(Float.NaN_))
    assertEquals(b_float(-1.0), p_byte(-1) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), p_byte(-1) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(23) % b_float(0.0))
    assertEquals(b_float(0.0), b_byte(23) % b_float(1.0))
    assertEquals(b_float(0.0), b_byte(23) % b_float(-1.0))
    assertEquals(b_float(0.0), b_byte(23) % b_float(23.0))
    assertEquals(b_float(23.0), b_byte(23) % b_float(23.123))
    assertEquals(b_float(23.0), b_byte(23) % b_float(-32.0))
    assertEquals(b_float(23.0), b_byte(23) % b_float(-32.456))
    assertEquals(b_float(23.0), b_byte(23) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(23.0), b_byte(23) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(23) % b_float(Float.NaN_))
    assertEquals(b_float(23.0), b_byte(23) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(23) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(-32) % b_float(0.0))
    assertEquals(b_float(-0.0), b_byte(-32) % b_float(1.0))
    assertEquals(b_float(-0.0), b_byte(-32) % b_float(-1.0))
    assertEquals(b_float(-9.0), b_byte(-32) % b_float(23.0))
    assertEquals(b_float(-8.877001), b_byte(-32) % b_float(23.123))
    assertEquals(b_float(-0.0), b_byte(-32) % b_float(-32.0))
    assertEquals(b_float(-32.0), b_byte(-32) % b_float(-32.456))
    assertEquals(b_float(-32.0), b_byte(-32) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-32.0), b_byte(-32) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(-32) % b_float(Float.NaN_))
    assertEquals(b_float(-32.0), b_byte(-32) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), b_byte(-32) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(Byte.MAX_VALUE) % b_float(0.0))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % b_float(1.0))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % b_float(-1.0))
    assertEquals(b_float(12.0), b_byte(Byte.MAX_VALUE) % b_float(23.0))
    assertEquals(b_float(11.385004), b_byte(Byte.MAX_VALUE) % b_float(23.123))
    assertEquals(b_float(31.0), b_byte(Byte.MAX_VALUE) % b_float(-32.0))
    assertEquals(b_float(29.631996), b_byte(Byte.MAX_VALUE) % b_float(-32.456))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(Byte.MAX_VALUE) % b_float(Float.NaN_))
    assertEquals(b_float(127.0), b_byte(Byte.MAX_VALUE) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), b_byte(Byte.MAX_VALUE) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), b_byte(Byte.MIN_VALUE) % b_float(0.0))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % b_float(1.0))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % b_float(-1.0))
    assertEquals(b_float(-13.0), b_byte(Byte.MIN_VALUE) % b_float(23.0))
    assertEquals(b_float(-12.385004), b_byte(Byte.MIN_VALUE) % b_float(23.123))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % b_float(-32.0))
    assertEquals(b_float(-30.631996), b_byte(Byte.MIN_VALUE) % b_float(-32.456))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), b_byte(Byte.MIN_VALUE) % b_float(Float.NaN_))
    assertEquals(b_float(-128.0), b_byte(Byte.MIN_VALUE) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), b_byte(Byte.MIN_VALUE) % b_float(Float.MIN_VALUE))

    assertEquals(Float, statictypeof(b_byte(0) % b_float(0.0)))
  }

  function testBytePDoubleRemainder() {
    assertEquals(b_double(Double.NaN_), b_byte(0) % p_double(0.0))
    assertEquals(b_double(0.0), b_byte(0) % p_double(1.0))
    assertEquals(b_double(0.0), b_byte(0) % p_double(-1.0))
    assertEquals(b_double(0.0), b_byte(0) % p_double(23.0))
    assertEquals(b_double(0.0), b_byte(0) % p_double(23.123))
    assertEquals(b_double(0.0), b_byte(0) % p_double(-32.0))
    assertEquals(b_double(0.0), b_byte(0) % p_double(-32.456))
    assertEquals(b_double(0.0), b_byte(0) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(0.0), b_byte(0) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(0) % p_double(Double.NaN_))
    assertEquals(b_double(0.0), b_byte(0) % p_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(0) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_byte(1) % p_double(0.0))
    assertEquals(p_double(0.0), p_byte(1) % p_double(1.0))
    assertEquals(p_double(0.0), p_byte(1) % p_double(-1.0))
    assertEquals(p_double(1.0), p_byte(1) % p_double(23.0))
    assertEquals(p_double(1.0), p_byte(1) % p_double(23.123))
    assertEquals(p_double(1.0), p_byte(1) % p_double(-32.0))
    assertEquals(p_double(1.0), p_byte(1) % p_double(-32.456))
    assertEquals(p_double(1.0), p_byte(1) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(1.0), p_byte(1) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_byte(1) % p_double(Double.NaN_))
    assertEquals(p_double(1.0), p_byte(1) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(0.0), p_byte(1) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_byte(-1) % p_double(0.0))
    assertEquals(p_double(-0.0), p_byte(-1) % p_double(1.0))
    assertEquals(p_double(-0.0), p_byte(-1) % p_double(-1.0))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(23.0))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(23.123))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(-32.0))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(-32.456))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_byte(-1) % p_double(Double.NaN_))
    assertEquals(p_double(-1.0), p_byte(-1) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(-0.0), p_byte(-1) % p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(23) % p_double(0.0))
    assertEquals(b_double(0.0), b_byte(23) % p_double(1.0))
    assertEquals(b_double(0.0), b_byte(23) % p_double(-1.0))
    assertEquals(b_double(0.0), b_byte(23) % p_double(23.0))
    assertEquals(b_double(23.0), b_byte(23) % p_double(23.123))
    assertEquals(b_double(23.0), b_byte(23) % p_double(-32.0))
    assertEquals(b_double(23.0), b_byte(23) % p_double(-32.456))
    assertEquals(b_double(23.0), b_byte(23) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(23.0), b_byte(23) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(23) % p_double(Double.NaN_))
    assertEquals(b_double(23.0), b_byte(23) % p_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(23) % p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(-32) % p_double(0.0))
    assertEquals(b_double(-0.0), b_byte(-32) % p_double(1.0))
    assertEquals(b_double(-0.0), b_byte(-32) % p_double(-1.0))
    assertEquals(b_double(-9.0), b_byte(-32) % p_double(23.0))
    assertEquals(b_double(-8.876999999999999), b_byte(-32) % p_double(23.123))
    assertEquals(b_double(-0.0), b_byte(-32) % p_double(-32.0))
    assertEquals(b_double(-32.0), b_byte(-32) % p_double(-32.456))
    assertEquals(b_double(-32.0), b_byte(-32) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-32.0), b_byte(-32) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(-32) % p_double(Double.NaN_))
    assertEquals(b_double(-32.0), b_byte(-32) % p_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), b_byte(-32) % p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(Byte.MAX_VALUE) % p_double(0.0))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % p_double(1.0))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % p_double(-1.0))
    assertEquals(b_double(12.0), b_byte(Byte.MAX_VALUE) % p_double(23.0))
    assertEquals(b_double(11.384999999999994), b_byte(Byte.MAX_VALUE) % p_double(23.123))
    assertEquals(b_double(31.0), b_byte(Byte.MAX_VALUE) % p_double(-32.0))
    assertEquals(b_double(29.63199999999999), b_byte(Byte.MAX_VALUE) % p_double(-32.456))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(Byte.MAX_VALUE) % p_double(Double.NaN_))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % p_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % p_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(Byte.MIN_VALUE) % p_double(0.0))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % p_double(1.0))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % p_double(-1.0))
    assertEquals(b_double(-13.0), b_byte(Byte.MIN_VALUE) % p_double(23.0))
    assertEquals(b_double(-12.384999999999994), b_byte(Byte.MIN_VALUE) % p_double(23.123))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % p_double(-32.0))
    assertEquals(b_double(-30.63199999999999), b_byte(Byte.MIN_VALUE) % p_double(-32.456))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(Byte.MIN_VALUE) % p_double(Double.NaN_))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % p_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % p_double(Double.MIN_VALUE))

    assertEquals(Double, statictypeof(b_byte(0) % p_double(0.0)))
  }

  function testByteDoubleRemainder() {
    assertEquals(b_double(Double.NaN_), b_byte(0) % b_double(0.0))
    assertEquals(b_double(0.0), b_byte(0) % b_double(1.0))
    assertEquals(b_double(0.0), b_byte(0) % b_double(-1.0))
    assertEquals(b_double(0.0), b_byte(0) % b_double(23.0))
    assertEquals(b_double(0.0), b_byte(0) % b_double(23.123))
    assertEquals(b_double(0.0), b_byte(0) % b_double(-32.0))
    assertEquals(b_double(0.0), b_byte(0) % b_double(-32.456))
    assertEquals(b_double(0.0), b_byte(0) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(0.0), b_byte(0) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(0) % b_double(Double.NaN_))
    assertEquals(b_double(0.0), b_byte(0) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(0) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_byte(1) % b_double(0.0))
    assertEquals(b_double(0.0), p_byte(1) % b_double(1.0))
    assertEquals(b_double(0.0), p_byte(1) % b_double(-1.0))
    assertEquals(b_double(1.0), p_byte(1) % b_double(23.0))
    assertEquals(b_double(1.0), p_byte(1) % b_double(23.123))
    assertEquals(b_double(1.0), p_byte(1) % b_double(-32.0))
    assertEquals(b_double(1.0), p_byte(1) % b_double(-32.456))
    assertEquals(b_double(1.0), p_byte(1) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(1.0), p_byte(1) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_byte(1) % b_double(Double.NaN_))
    assertEquals(b_double(1.0), p_byte(1) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), p_byte(1) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_byte(-1) % b_double(0.0))
    assertEquals(b_double(-0.0), p_byte(-1) % b_double(1.0))
    assertEquals(b_double(-0.0), p_byte(-1) % b_double(-1.0))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(23.0))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(23.123))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(-32.0))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(-32.456))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_byte(-1) % b_double(Double.NaN_))
    assertEquals(b_double(-1.0), p_byte(-1) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), p_byte(-1) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(23) % b_double(0.0))
    assertEquals(b_double(0.0), b_byte(23) % b_double(1.0))
    assertEquals(b_double(0.0), b_byte(23) % b_double(-1.0))
    assertEquals(b_double(0.0), b_byte(23) % b_double(23.0))
    assertEquals(b_double(23.0), b_byte(23) % b_double(23.123))
    assertEquals(b_double(23.0), b_byte(23) % b_double(-32.0))
    assertEquals(b_double(23.0), b_byte(23) % b_double(-32.456))
    assertEquals(b_double(23.0), b_byte(23) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(23.0), b_byte(23) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(23) % b_double(Double.NaN_))
    assertEquals(b_double(23.0), b_byte(23) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(23) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(-32) % b_double(0.0))
    assertEquals(b_double(-0.0), b_byte(-32) % b_double(1.0))
    assertEquals(b_double(-0.0), b_byte(-32) % b_double(-1.0))
    assertEquals(b_double(-9.0), b_byte(-32) % b_double(23.0))
    assertEquals(b_double(-8.876999999999999), b_byte(-32) % b_double(23.123))
    assertEquals(b_double(-0.0), b_byte(-32) % b_double(-32.0))
    assertEquals(b_double(-32.0), b_byte(-32) % b_double(-32.456))
    assertEquals(b_double(-32.0), b_byte(-32) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-32.0), b_byte(-32) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(-32) % b_double(Double.NaN_))
    assertEquals(b_double(-32.0), b_byte(-32) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), b_byte(-32) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(Byte.MAX_VALUE) % b_double(0.0))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % b_double(1.0))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % b_double(-1.0))
    assertEquals(b_double(12.0), b_byte(Byte.MAX_VALUE) % b_double(23.0))
    assertEquals(b_double(11.384999999999994), b_byte(Byte.MAX_VALUE) % b_double(23.123))
    assertEquals(b_double(31.0), b_byte(Byte.MAX_VALUE) % b_double(-32.0))
    assertEquals(b_double(29.63199999999999), b_byte(Byte.MAX_VALUE) % b_double(-32.456))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(Byte.MAX_VALUE) % b_double(Double.NaN_))
    assertEquals(b_double(127.0), b_byte(Byte.MAX_VALUE) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), b_byte(Byte.MAX_VALUE) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), b_byte(Byte.MIN_VALUE) % b_double(0.0))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % b_double(1.0))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % b_double(-1.0))
    assertEquals(b_double(-13.0), b_byte(Byte.MIN_VALUE) % b_double(23.0))
    assertEquals(b_double(-12.384999999999994), b_byte(Byte.MIN_VALUE) % b_double(23.123))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % b_double(-32.0))
    assertEquals(b_double(-30.63199999999999), b_byte(Byte.MIN_VALUE) % b_double(-32.456))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), b_byte(Byte.MIN_VALUE) % b_double(Double.NaN_))
    assertEquals(b_double(-128.0), b_byte(Byte.MIN_VALUE) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), b_byte(Byte.MIN_VALUE) % b_double(Double.MIN_VALUE))

    assertEquals(Double, statictypeof(b_byte(0) % b_double(0.0)))
  }

  function testByteBigIntegerRemainder() {
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(0) % big_int("0"))
    assertEquals(big_int("0"), b_byte(0) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(0) % big_int("-1"))
    assertEquals(big_int("0"), b_byte(0) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(0) % big_int("-32"))
    assertEquals(big_int("0"), b_byte(0) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(0) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(1) % big_int("0"))
    assertEquals(big_int("0"), p_byte(1) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(1) % big_int("-1"))
    assertEquals(big_int("1"), p_byte(1) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(1) % big_int("-32"))
    assertEquals(big_int("1"), p_byte(1) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(1) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(-1) % big_int("0"))
    assertEquals(big_int("0"), p_byte(-1) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(-1) % big_int("-1"))
    assertEquals(big_int("22"), p_byte(-1) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(-1) % big_int("-32"))
    assertEquals(big_int("123456789012345678901234567889"), p_byte(-1) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_byte(-1) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(23) % big_int("0"))
    assertEquals(big_int("0"), b_byte(23) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(23) % big_int("-1"))
    assertEquals(big_int("0"), b_byte(23) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(23) % big_int("-32"))
    assertEquals(big_int("23"), b_byte(23) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(23) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(-32) % big_int("0"))
    assertEquals(big_int("0"), b_byte(-32) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(-32) % big_int("-1"))
    assertEquals(big_int("14"), b_byte(-32) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(-32) % big_int("-32"))
    assertEquals(big_int("123456789012345678901234567858"), b_byte(-32) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(-32) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MAX_VALUE) % big_int("0"))
    assertEquals(big_int("0"), b_byte(Byte.MAX_VALUE) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MAX_VALUE) % big_int("-1"))
    assertEquals(big_int("12"), b_byte(Byte.MAX_VALUE) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MAX_VALUE) % big_int("-32"))
    assertEquals(big_int("127"), b_byte(Byte.MAX_VALUE) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MAX_VALUE) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MIN_VALUE) % big_int("0"))
    assertEquals(big_int("0"), b_byte(Byte.MIN_VALUE) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MIN_VALUE) % big_int("-1"))
    assertEquals(big_int("10"), b_byte(Byte.MIN_VALUE) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MIN_VALUE) % big_int("-32"))
    assertEquals(big_int("123456789012345678901234567762"), b_byte(Byte.MIN_VALUE) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> b_byte(Byte.MIN_VALUE) % big_int("-123456789012345678901234567890"))

    assertEquals(BigInteger, statictypeof(b_byte(0) % big_int("0")))
  }

  function testByteBigDecimalRemainder() {
    assertThrowsArithmeticException("Division undefined", \ -> b_byte(0) % big_decimal("0"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("1"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("-1"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("23"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("-32"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("-32.456"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("0"), b_byte(0) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_byte(1) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_byte(1) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_byte(1) % big_decimal("-1"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("23"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("23.123"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("-32"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("-32.456"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("1"), p_byte(1) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_byte(-1) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_byte(-1) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_byte(-1) % big_decimal("-1"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("23"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("23.123"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("-32"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("-32.456"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("1"), p_byte(-1) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> b_byte(23) % big_decimal("0"))
    assertEquals(big_decimal("0"), b_byte(23) % big_decimal("1"))
    assertEquals(big_decimal("0"), b_byte(23) % big_decimal("-1"))
    assertEquals(big_decimal("0"), b_byte(23) % big_decimal("23"))
    assertEquals(big_decimal("23"), b_byte(23) % big_decimal("23.123"))
    assertEquals(big_decimal("23"), b_byte(23) % big_decimal("-32"))
    assertEquals(big_decimal("23"), b_byte(23) % big_decimal("-32.456"))
    assertEquals(big_decimal("23"), b_byte(23) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("23"), b_byte(23) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> b_byte(-32) % big_decimal("0"))
    assertEquals(big_decimal("0"), b_byte(-32) % big_decimal("1"))
    assertEquals(big_decimal("0"), b_byte(-32) % big_decimal("-1"))
    assertEquals(big_decimal("9"), b_byte(-32) % big_decimal("23"))
    assertEquals(big_decimal("8.877"), b_byte(-32) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), b_byte(-32) % big_decimal("-32"))
    assertEquals(big_decimal("32"), b_byte(-32) % big_decimal("-32.456"))
    assertEquals(big_decimal("32"), b_byte(-32) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("32"), b_byte(-32) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> b_byte(Byte.MAX_VALUE) % big_decimal("0"))
    assertEquals(big_decimal("0"), b_byte(Byte.MAX_VALUE) % big_decimal("1"))
    assertEquals(big_decimal("0"), b_byte(Byte.MAX_VALUE) % big_decimal("-1"))
    assertEquals(big_decimal("12"), b_byte(Byte.MAX_VALUE) % big_decimal("23"))
    assertEquals(big_decimal("11.385"), b_byte(Byte.MAX_VALUE) % big_decimal("23.123"))
    assertEquals(big_decimal("31"), b_byte(Byte.MAX_VALUE) % big_decimal("-32"))
    assertEquals(big_decimal("29.632"), b_byte(Byte.MAX_VALUE) % big_decimal("-32.456"))
    assertEquals(big_decimal("127"), b_byte(Byte.MAX_VALUE) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("127"), b_byte(Byte.MAX_VALUE) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> b_byte(Byte.MIN_VALUE) % big_decimal("0"))
    assertEquals(big_decimal("0"), b_byte(Byte.MIN_VALUE) % big_decimal("1"))
    assertEquals(big_decimal("0"), b_byte(Byte.MIN_VALUE) % big_decimal("-1"))
    assertEquals(big_decimal("13"), b_byte(Byte.MIN_VALUE) % big_decimal("23"))
    assertEquals(big_decimal("12.385"), b_byte(Byte.MIN_VALUE) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), b_byte(Byte.MIN_VALUE) % big_decimal("-32"))
    assertEquals(big_decimal("30.632"), b_byte(Byte.MIN_VALUE) % big_decimal("-32.456"))
    assertEquals(big_decimal("128"), b_byte(Byte.MIN_VALUE) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("128"), b_byte(Byte.MIN_VALUE) % big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(BigDecimal, statictypeof(b_byte(0) % big_decimal("0")))
  }

}

