package gw.spec.core.expressions.arithmetic.remainder
uses java.lang.Byte
uses java.lang.Character
uses java.lang.Integer
uses java.lang.Short
uses java.lang.Long
uses java.lang.Float
uses java.lang.Double
uses java.math.BigInteger
uses java.math.BigDecimal
uses gw.spec.core.expressions.arithmetic.ArithmeticTestBase

class Remainder_PLongTest extends ArithmeticTestBase {

  function testPLongPByteRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % p_byte(0))
    assertEquals(p_long(0), p_long(0) % p_byte(1))
    assertEquals(p_long(0), p_long(0) % p_byte(-1))
    assertEquals(p_long(0), p_long(0) % p_byte(23))
    assertEquals(p_long(0), p_long(0) % p_byte(-32))
    assertEquals(p_long(0), p_long(0) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(0), p_long(0) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % p_byte(0))
    assertEquals(p_long(0), p_long(1) % p_byte(1))
    assertEquals(p_long(0), p_long(1) % p_byte(-1))
    assertEquals(p_long(1), p_long(1) % p_byte(23))
    assertEquals(p_long(1), p_long(1) % p_byte(-32))
    assertEquals(p_long(1), p_long(1) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(1), p_long(1) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % p_byte(0))
    assertEquals(p_long(0), p_long(-1) % p_byte(1))
    assertEquals(p_long(0), p_long(-1) % p_byte(-1))
    assertEquals(p_long(-1), p_long(-1) % p_byte(23))
    assertEquals(p_long(-1), p_long(-1) % p_byte(-32))
    assertEquals(p_long(-1), p_long(-1) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(-1), p_long(-1) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % p_byte(0))
    assertEquals(p_long(0), p_long(23) % p_byte(1))
    assertEquals(p_long(0), p_long(23) % p_byte(-1))
    assertEquals(p_long(0), p_long(23) % p_byte(23))
    assertEquals(p_long(23), p_long(23) % p_byte(-32))
    assertEquals(p_long(23), p_long(23) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(23), p_long(23) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % p_byte(0))
    assertEquals(p_long(0), p_long(-32) % p_byte(1))
    assertEquals(p_long(0), p_long(-32) % p_byte(-1))
    assertEquals(p_long(-9), p_long(-32) % p_byte(23))
    assertEquals(p_long(0), p_long(-32) % p_byte(-32))
    assertEquals(p_long(-32), p_long(-32) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(-32), p_long(-32) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % p_byte(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_byte(1))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_byte(-1))
    assertEquals(p_long(2), p_long(Long.MAX_VALUE) % p_byte(23))
    assertEquals(p_long(31), p_long(Long.MAX_VALUE) % p_byte(-32))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(127), p_long(Long.MAX_VALUE) % p_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % p_byte(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(1))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(-1))
    assertEquals(p_long(-3), p_long(Long.MIN_VALUE) % p_byte(23))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(-32))
    assertEquals(p_long(-1), p_long(Long.MIN_VALUE) % p_byte(Byte.MAX_VALUE))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(Byte.MIN_VALUE))

    assertEquals(long, statictypeof(p_long(0) % p_byte(0)))
  }

  function testPLongByteRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % b_byte(0))
    assertEquals(p_long(0), p_long(0) % p_byte(1))
    assertEquals(p_long(0), p_long(0) % p_byte(-1))
    assertEquals(b_long(0), p_long(0) % b_byte(23))
    assertEquals(b_long(0), p_long(0) % b_byte(-32))
    assertEquals(b_long(0), p_long(0) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(0), p_long(0) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % b_byte(0))
    assertEquals(p_long(0), p_long(1) % p_byte(1))
    assertEquals(p_long(0), p_long(1) % p_byte(-1))
    assertEquals(b_long(1), p_long(1) % b_byte(23))
    assertEquals(b_long(1), p_long(1) % b_byte(-32))
    assertEquals(b_long(1), p_long(1) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(1), p_long(1) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % b_byte(0))
    assertEquals(p_long(0), p_long(-1) % p_byte(1))
    assertEquals(p_long(0), p_long(-1) % p_byte(-1))
    assertEquals(b_long(-1), p_long(-1) % b_byte(23))
    assertEquals(b_long(-1), p_long(-1) % b_byte(-32))
    assertEquals(b_long(-1), p_long(-1) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(-1), p_long(-1) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % b_byte(0))
    assertEquals(p_long(0), p_long(23) % p_byte(1))
    assertEquals(p_long(0), p_long(23) % p_byte(-1))
    assertEquals(b_long(0), p_long(23) % b_byte(23))
    assertEquals(b_long(23), p_long(23) % b_byte(-32))
    assertEquals(b_long(23), p_long(23) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(23), p_long(23) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % b_byte(0))
    assertEquals(p_long(0), p_long(-32) % p_byte(1))
    assertEquals(p_long(0), p_long(-32) % p_byte(-1))
    assertEquals(b_long(-9), p_long(-32) % b_byte(23))
    assertEquals(b_long(0), p_long(-32) % b_byte(-32))
    assertEquals(b_long(-32), p_long(-32) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(-32), p_long(-32) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % b_byte(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_byte(1))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_byte(-1))
    assertEquals(b_long(2), p_long(Long.MAX_VALUE) % b_byte(23))
    assertEquals(b_long(31), p_long(Long.MAX_VALUE) % b_byte(-32))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(127), p_long(Long.MAX_VALUE) % b_byte(Byte.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % b_byte(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(1))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_byte(-1))
    assertEquals(b_long(-3), p_long(Long.MIN_VALUE) % b_byte(23))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_byte(-32))
    assertEquals(b_long(-1), p_long(Long.MIN_VALUE) % b_byte(Byte.MAX_VALUE))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_byte(Byte.MIN_VALUE))

    assertEquals(Long, statictypeof(p_long(0) % b_byte(0)))
  }

  function testPLongPShortRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % p_short(0))
    assertEquals(p_long(0), p_long(0) % p_short(1))
    assertEquals(p_long(0), p_long(0) % p_short(-1))
    assertEquals(p_long(0), p_long(0) % p_short(23))
    assertEquals(p_long(0), p_long(0) % p_short(-32))
    assertEquals(p_long(0), p_long(0) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(0), p_long(0) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % p_short(0))
    assertEquals(p_long(0), p_long(1) % p_short(1))
    assertEquals(p_long(0), p_long(1) % p_short(-1))
    assertEquals(p_long(1), p_long(1) % p_short(23))
    assertEquals(p_long(1), p_long(1) % p_short(-32))
    assertEquals(p_long(1), p_long(1) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(1), p_long(1) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % p_short(0))
    assertEquals(p_long(0), p_long(-1) % p_short(1))
    assertEquals(p_long(0), p_long(-1) % p_short(-1))
    assertEquals(p_long(-1), p_long(-1) % p_short(23))
    assertEquals(p_long(-1), p_long(-1) % p_short(-32))
    assertEquals(p_long(-1), p_long(-1) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(-1), p_long(-1) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % p_short(0))
    assertEquals(p_long(0), p_long(23) % p_short(1))
    assertEquals(p_long(0), p_long(23) % p_short(-1))
    assertEquals(p_long(0), p_long(23) % p_short(23))
    assertEquals(p_long(23), p_long(23) % p_short(-32))
    assertEquals(p_long(23), p_long(23) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(23), p_long(23) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % p_short(0))
    assertEquals(p_long(0), p_long(-32) % p_short(1))
    assertEquals(p_long(0), p_long(-32) % p_short(-1))
    assertEquals(p_long(-9), p_long(-32) % p_short(23))
    assertEquals(p_long(0), p_long(-32) % p_short(-32))
    assertEquals(p_long(-32), p_long(-32) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(-32), p_long(-32) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % p_short(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_short(1))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_short(-1))
    assertEquals(p_long(2), p_long(Long.MAX_VALUE) % p_short(23))
    assertEquals(p_long(31), p_long(Long.MAX_VALUE) % p_short(-32))
    assertEquals(p_long(7), p_long(Long.MAX_VALUE) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(32767), p_long(Long.MAX_VALUE) % p_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % p_short(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_short(1))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_short(-1))
    assertEquals(p_long(-3), p_long(Long.MIN_VALUE) % p_short(23))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_short(-32))
    assertEquals(p_long(-8), p_long(Long.MIN_VALUE) % p_short(Short.MAX_VALUE))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_short(Short.MIN_VALUE))

    assertEquals(long, statictypeof(p_long(0) % p_short(0)))
  }

  function testPLongShortRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % b_short(0))
    assertEquals(b_long(0), p_long(0) % b_short(1))
    assertEquals(b_long(0), p_long(0) % b_short(-1))
    assertEquals(b_long(0), p_long(0) % b_short(23))
    assertEquals(b_long(0), p_long(0) % b_short(-32))
    assertEquals(b_long(0), p_long(0) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(0), p_long(0) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % b_short(0))
    assertEquals(b_long(0), p_long(1) % b_short(1))
    assertEquals(b_long(0), p_long(1) % b_short(-1))
    assertEquals(b_long(1), p_long(1) % b_short(23))
    assertEquals(b_long(1), p_long(1) % b_short(-32))
    assertEquals(b_long(1), p_long(1) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(1), p_long(1) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % b_short(0))
    assertEquals(b_long(0), p_long(-1) % b_short(1))
    assertEquals(b_long(0), p_long(-1) % b_short(-1))
    assertEquals(b_long(-1), p_long(-1) % b_short(23))
    assertEquals(b_long(-1), p_long(-1) % b_short(-32))
    assertEquals(b_long(-1), p_long(-1) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(-1), p_long(-1) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % b_short(0))
    assertEquals(b_long(0), p_long(23) % b_short(1))
    assertEquals(b_long(0), p_long(23) % b_short(-1))
    assertEquals(b_long(0), p_long(23) % b_short(23))
    assertEquals(b_long(23), p_long(23) % b_short(-32))
    assertEquals(b_long(23), p_long(23) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(23), p_long(23) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % b_short(0))
    assertEquals(b_long(0), p_long(-32) % b_short(1))
    assertEquals(b_long(0), p_long(-32) % b_short(-1))
    assertEquals(b_long(-9), p_long(-32) % b_short(23))
    assertEquals(b_long(0), p_long(-32) % b_short(-32))
    assertEquals(b_long(-32), p_long(-32) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(-32), p_long(-32) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % b_short(0))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_short(1))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_short(-1))
    assertEquals(b_long(2), p_long(Long.MAX_VALUE) % b_short(23))
    assertEquals(b_long(31), p_long(Long.MAX_VALUE) % b_short(-32))
    assertEquals(b_long(7), p_long(Long.MAX_VALUE) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(32767), p_long(Long.MAX_VALUE) % b_short(Short.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % b_short(0))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_short(1))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_short(-1))
    assertEquals(b_long(-3), p_long(Long.MIN_VALUE) % b_short(23))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_short(-32))
    assertEquals(b_long(-8), p_long(Long.MIN_VALUE) % b_short(Short.MAX_VALUE))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_short(Short.MIN_VALUE))

    assertEquals(Long, statictypeof(p_long(0) % b_short(0)))
  }

  function testPLongPCharRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % p_char(0))
    assertEquals(p_long(0), p_long(0) % p_char(1))
    assertEquals(p_long(0), p_long(0) % p_char(23))
    assertEquals(p_long(0), p_long(0) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % p_char(0))
    assertEquals(p_long(0), p_long(1) % p_char(1))
    assertEquals(p_long(1), p_long(1) % p_char(23))
    assertEquals(p_long(1), p_long(1) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % p_char(0))
    assertEquals(p_long(0), p_long(-1) % p_char(1))
    assertEquals(p_long(-1), p_long(-1) % p_char(23))
    assertEquals(p_long(-1), p_long(-1) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % p_char(0))
    assertEquals(p_long(0), p_long(23) % p_char(1))
    assertEquals(p_long(0), p_long(23) % p_char(23))
    assertEquals(p_long(23), p_long(23) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % p_char(0))
    assertEquals(p_long(0), p_long(-32) % p_char(1))
    assertEquals(p_long(-9), p_long(-32) % p_char(23))
    assertEquals(p_long(-32), p_long(-32) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % p_char(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_char(1))
    assertEquals(p_long(2), p_long(Long.MAX_VALUE) % p_char(23))
    assertEquals(p_long(32767), p_long(Long.MAX_VALUE) % p_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % p_char(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_char(1))
    assertEquals(p_long(-3), p_long(Long.MIN_VALUE) % p_char(23))
    assertEquals(p_long(-32768), p_long(Long.MIN_VALUE) % p_char(Character.MAX_VALUE))

    assertEquals(long, statictypeof(p_long(0) % p_char(0)))
  }

  function testPLongCharacterRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % b_char(0))
    assertEquals(b_long(0), p_long(0) % b_char(1))
    assertEquals(b_long(0), p_long(0) % b_char(23))
    assertEquals(b_long(0), p_long(0) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % b_char(0))
    assertEquals(b_long(0), p_long(1) % b_char(1))
    assertEquals(b_long(1), p_long(1) % b_char(23))
    assertEquals(b_long(1), p_long(1) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % b_char(0))
    assertEquals(b_long(0), p_long(-1) % b_char(1))
    assertEquals(b_long(-1), p_long(-1) % b_char(23))
    assertEquals(b_long(-1), p_long(-1) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % b_char(0))
    assertEquals(b_long(0), p_long(23) % b_char(1))
    assertEquals(b_long(0), p_long(23) % b_char(23))
    assertEquals(b_long(23), p_long(23) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % b_char(0))
    assertEquals(b_long(0), p_long(-32) % b_char(1))
    assertEquals(b_long(-9), p_long(-32) % b_char(23))
    assertEquals(b_long(-32), p_long(-32) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % b_char(0))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_char(1))
    assertEquals(b_long(2), p_long(Long.MAX_VALUE) % b_char(23))
    assertEquals(b_long(32767), p_long(Long.MAX_VALUE) % b_char(Character.MAX_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % b_char(0))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_char(1))
    assertEquals(b_long(-3), p_long(Long.MIN_VALUE) % b_char(23))
    assertEquals(b_long(-32768), p_long(Long.MIN_VALUE) % b_char(Character.MAX_VALUE))

    assertEquals(Long, statictypeof(p_long(0) % b_char(0)))
  }

  function testPLongPIntRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % p_int(0))
    assertEquals(p_long(0), p_long(0) % p_int(1))
    assertEquals(p_long(0), p_long(0) % p_int(-1))
    assertEquals(p_long(0), p_long(0) % p_int(23))
    assertEquals(p_long(0), p_long(0) % p_int(-32))
    assertEquals(p_long(0), p_long(0) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(0), p_long(0) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % p_int(0))
    assertEquals(p_long(0), p_long(1) % p_int(1))
    assertEquals(p_long(0), p_long(1) % p_int(-1))
    assertEquals(p_long(1), p_long(1) % p_int(23))
    assertEquals(p_long(1), p_long(1) % p_int(-32))
    assertEquals(p_long(1), p_long(1) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(1), p_long(1) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % p_int(0))
    assertEquals(p_long(0), p_long(-1) % p_int(1))
    assertEquals(p_long(0), p_long(-1) % p_int(-1))
    assertEquals(p_long(-1), p_long(-1) % p_int(23))
    assertEquals(p_long(-1), p_long(-1) % p_int(-32))
    assertEquals(p_long(-1), p_long(-1) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(-1), p_long(-1) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % p_int(0))
    assertEquals(p_long(0), p_long(23) % p_int(1))
    assertEquals(p_long(0), p_long(23) % p_int(-1))
    assertEquals(p_long(0), p_long(23) % p_int(23))
    assertEquals(p_long(23), p_long(23) % p_int(-32))
    assertEquals(p_long(23), p_long(23) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(23), p_long(23) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % p_int(0))
    assertEquals(p_long(0), p_long(-32) % p_int(1))
    assertEquals(p_long(0), p_long(-32) % p_int(-1))
    assertEquals(p_long(-9), p_long(-32) % p_int(23))
    assertEquals(p_long(0), p_long(-32) % p_int(-32))
    assertEquals(p_long(-32), p_long(-32) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(-32), p_long(-32) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % p_int(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_int(1))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_int(-1))
    assertEquals(p_long(2), p_long(Long.MAX_VALUE) % p_int(23))
    assertEquals(p_long(31), p_long(Long.MAX_VALUE) % p_int(-32))
    assertEquals(p_long(1), p_long(Long.MAX_VALUE) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(2147483647), p_long(Long.MAX_VALUE) % p_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % p_int(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_int(1))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_int(-1))
    assertEquals(p_long(-3), p_long(Long.MIN_VALUE) % p_int(23))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_int(-32))
    assertEquals(p_long(-2), p_long(Long.MIN_VALUE) % p_int(Integer.MAX_VALUE))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_int(Integer.MIN_VALUE))

    assertEquals(long, statictypeof(p_long(0) % p_int(0)))
  }

  function testPLongIntegerRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % b_int(0))
    assertEquals(b_long(0), p_long(0) % b_int(1))
    assertEquals(b_long(0), p_long(0) % b_int(-1))
    assertEquals(b_long(0), p_long(0) % b_int(23))
    assertEquals(b_long(0), p_long(0) % b_int(-32))
    assertEquals(b_long(0), p_long(0) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(0), p_long(0) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % b_int(0))
    assertEquals(b_long(0), p_long(1) % b_int(1))
    assertEquals(b_long(0), p_long(1) % b_int(-1))
    assertEquals(b_long(1), p_long(1) % b_int(23))
    assertEquals(b_long(1), p_long(1) % b_int(-32))
    assertEquals(b_long(1), p_long(1) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(1), p_long(1) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % b_int(0))
    assertEquals(b_long(0), p_long(-1) % b_int(1))
    assertEquals(b_long(0), p_long(-1) % b_int(-1))
    assertEquals(b_long(-1), p_long(-1) % b_int(23))
    assertEquals(b_long(-1), p_long(-1) % b_int(-32))
    assertEquals(b_long(-1), p_long(-1) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(-1), p_long(-1) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % b_int(0))
    assertEquals(b_long(0), p_long(23) % b_int(1))
    assertEquals(b_long(0), p_long(23) % b_int(-1))
    assertEquals(b_long(0), p_long(23) % b_int(23))
    assertEquals(b_long(23), p_long(23) % b_int(-32))
    assertEquals(b_long(23), p_long(23) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(23), p_long(23) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % b_int(0))
    assertEquals(b_long(0), p_long(-32) % b_int(1))
    assertEquals(b_long(0), p_long(-32) % b_int(-1))
    assertEquals(b_long(-9), p_long(-32) % b_int(23))
    assertEquals(b_long(0), p_long(-32) % b_int(-32))
    assertEquals(b_long(-32), p_long(-32) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(-32), p_long(-32) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % b_int(0))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_int(1))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_int(-1))
    assertEquals(b_long(2), p_long(Long.MAX_VALUE) % b_int(23))
    assertEquals(b_long(31), p_long(Long.MAX_VALUE) % b_int(-32))
    assertEquals(b_long(1), p_long(Long.MAX_VALUE) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(2147483647), p_long(Long.MAX_VALUE) % b_int(Integer.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % b_int(0))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_int(1))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_int(-1))
    assertEquals(b_long(-3), p_long(Long.MIN_VALUE) % b_int(23))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_int(-32))
    assertEquals(b_long(-2), p_long(Long.MIN_VALUE) % b_int(Integer.MAX_VALUE))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_int(Integer.MIN_VALUE))

    assertEquals(Long, statictypeof(p_long(0) % b_int(0)))
  }

  function testPLongPLongRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % p_long(0))
    assertEquals(p_long(0), p_long(0) % p_long(1))
    assertEquals(p_long(0), p_long(0) % p_long(-1))
    assertEquals(p_long(0), p_long(0) % p_long(23))
    assertEquals(p_long(0), p_long(0) % p_long(-32))
    assertEquals(p_long(0), p_long(0) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(0), p_long(0) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % p_long(0))
    assertEquals(p_long(0), p_long(1) % p_long(1))
    assertEquals(p_long(0), p_long(1) % p_long(-1))
    assertEquals(p_long(1), p_long(1) % p_long(23))
    assertEquals(p_long(1), p_long(1) % p_long(-32))
    assertEquals(p_long(1), p_long(1) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(1), p_long(1) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % p_long(0))
    assertEquals(p_long(0), p_long(-1) % p_long(1))
    assertEquals(p_long(0), p_long(-1) % p_long(-1))
    assertEquals(p_long(-1), p_long(-1) % p_long(23))
    assertEquals(p_long(-1), p_long(-1) % p_long(-32))
    assertEquals(p_long(-1), p_long(-1) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(-1), p_long(-1) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % p_long(0))
    assertEquals(p_long(0), p_long(23) % p_long(1))
    assertEquals(p_long(0), p_long(23) % p_long(-1))
    assertEquals(p_long(0), p_long(23) % p_long(23))
    assertEquals(p_long(23), p_long(23) % p_long(-32))
    assertEquals(p_long(23), p_long(23) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(23), p_long(23) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % p_long(0))
    assertEquals(p_long(0), p_long(-32) % p_long(1))
    assertEquals(p_long(0), p_long(-32) % p_long(-1))
    assertEquals(p_long(-9), p_long(-32) % p_long(23))
    assertEquals(p_long(0), p_long(-32) % p_long(-32))
    assertEquals(p_long(-32), p_long(-32) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(-32), p_long(-32) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % p_long(0))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_long(1))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_long(-1))
    assertEquals(p_long(2), p_long(Long.MAX_VALUE) % p_long(23))
    assertEquals(p_long(31), p_long(Long.MAX_VALUE) % p_long(-32))
    assertEquals(p_long(0), p_long(Long.MAX_VALUE) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(9223372036854775807), p_long(Long.MAX_VALUE) % p_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % p_long(0))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_long(1))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_long(-1))
    assertEquals(p_long(-3), p_long(Long.MIN_VALUE) % p_long(23))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_long(-32))
    assertEquals(p_long(-1), p_long(Long.MIN_VALUE) % p_long(Long.MAX_VALUE))
    assertEquals(p_long(0), p_long(Long.MIN_VALUE) % p_long(Long.MIN_VALUE))

    assertEquals(long, statictypeof(p_long(0) % p_long(0)))
  }

  function testPLongLongRemainder() {
    assertThrowsArithmeticException("/ by zero", \ -> p_long(0) % b_long(0))
    assertEquals(b_long(0), p_long(0) % b_long(1))
    assertEquals(b_long(0), p_long(0) % b_long(-1))
    assertEquals(b_long(0), p_long(0) % b_long(23))
    assertEquals(b_long(0), p_long(0) % b_long(-32))
    assertEquals(b_long(0), p_long(0) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(0), p_long(0) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(1) % b_long(0))
    assertEquals(b_long(0), p_long(1) % b_long(1))
    assertEquals(b_long(0), p_long(1) % b_long(-1))
    assertEquals(b_long(1), p_long(1) % b_long(23))
    assertEquals(b_long(1), p_long(1) % b_long(-32))
    assertEquals(b_long(1), p_long(1) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(1), p_long(1) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-1) % b_long(0))
    assertEquals(b_long(0), p_long(-1) % b_long(1))
    assertEquals(b_long(0), p_long(-1) % b_long(-1))
    assertEquals(b_long(-1), p_long(-1) % b_long(23))
    assertEquals(b_long(-1), p_long(-1) % b_long(-32))
    assertEquals(b_long(-1), p_long(-1) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(-1), p_long(-1) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(23) % b_long(0))
    assertEquals(b_long(0), p_long(23) % b_long(1))
    assertEquals(b_long(0), p_long(23) % b_long(-1))
    assertEquals(b_long(0), p_long(23) % b_long(23))
    assertEquals(b_long(23), p_long(23) % b_long(-32))
    assertEquals(b_long(23), p_long(23) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(23), p_long(23) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(-32) % b_long(0))
    assertEquals(b_long(0), p_long(-32) % b_long(1))
    assertEquals(b_long(0), p_long(-32) % b_long(-1))
    assertEquals(b_long(-9), p_long(-32) % b_long(23))
    assertEquals(b_long(0), p_long(-32) % b_long(-32))
    assertEquals(b_long(-32), p_long(-32) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(-32), p_long(-32) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MAX_VALUE) % b_long(0))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_long(1))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_long(-1))
    assertEquals(b_long(2), p_long(Long.MAX_VALUE) % b_long(23))
    assertEquals(b_long(31), p_long(Long.MAX_VALUE) % b_long(-32))
    assertEquals(b_long(0), p_long(Long.MAX_VALUE) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(9223372036854775807), p_long(Long.MAX_VALUE) % b_long(Long.MIN_VALUE))

    assertThrowsArithmeticException("/ by zero", \ -> p_long(Long.MIN_VALUE) % b_long(0))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_long(1))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_long(-1))
    assertEquals(b_long(-3), p_long(Long.MIN_VALUE) % b_long(23))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_long(-32))
    assertEquals(b_long(-1), p_long(Long.MIN_VALUE) % b_long(Long.MAX_VALUE))
    assertEquals(b_long(0), p_long(Long.MIN_VALUE) % b_long(Long.MIN_VALUE))

    assertEquals(Long, statictypeof(p_long(0) % b_long(0)))
  }

  function testPLongPFloatRemainder() {
    assertEquals(p_float(Float.NaN_), p_long(0) % p_float(0.0))
    assertEquals(p_float(0.0), p_long(0) % p_float(1.0))
    assertEquals(p_float(0.0), p_long(0) % p_float(-1.0))
    assertEquals(p_float(0.0), p_long(0) % p_float(23.0))
    assertEquals(p_float(0.0), p_long(0) % p_float(23.123))
    assertEquals(p_float(0.0), p_long(0) % p_float(-32.0))
    assertEquals(p_float(0.0), p_long(0) % p_float(-32.456))
    assertEquals(p_float(0.0), p_long(0) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(0.0), p_long(0) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(0) % p_float(Float.NaN_))
    assertEquals(p_float(0.0), p_long(0) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(0.0), p_long(0) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(1) % p_float(0.0))
    assertEquals(p_float(0.0), p_long(1) % p_float(1.0))
    assertEquals(p_float(0.0), p_long(1) % p_float(-1.0))
    assertEquals(p_float(1.0), p_long(1) % p_float(23.0))
    assertEquals(p_float(1.0), p_long(1) % p_float(23.123))
    assertEquals(p_float(1.0), p_long(1) % p_float(-32.0))
    assertEquals(p_float(1.0), p_long(1) % p_float(-32.456))
    assertEquals(p_float(1.0), p_long(1) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(1.0), p_long(1) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(1) % p_float(Float.NaN_))
    assertEquals(p_float(1.0), p_long(1) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(0.0), p_long(1) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(-1) % p_float(0.0))
    assertEquals(p_float(-0.0), p_long(-1) % p_float(1.0))
    assertEquals(p_float(-0.0), p_long(-1) % p_float(-1.0))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(23.0))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(23.123))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(-32.0))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(-32.456))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(-1) % p_float(Float.NaN_))
    assertEquals(p_float(-1.0), p_long(-1) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(-0.0), p_long(-1) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(23) % p_float(0.0))
    assertEquals(p_float(0.0), p_long(23) % p_float(1.0))
    assertEquals(p_float(0.0), p_long(23) % p_float(-1.0))
    assertEquals(p_float(0.0), p_long(23) % p_float(23.0))
    assertEquals(p_float(23.0), p_long(23) % p_float(23.123))
    assertEquals(p_float(23.0), p_long(23) % p_float(-32.0))
    assertEquals(p_float(23.0), p_long(23) % p_float(-32.456))
    assertEquals(p_float(23.0), p_long(23) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(23.0), p_long(23) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(23) % p_float(Float.NaN_))
    assertEquals(p_float(23.0), p_long(23) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(0.0), p_long(23) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(-32) % p_float(0.0))
    assertEquals(p_float(-0.0), p_long(-32) % p_float(1.0))
    assertEquals(p_float(-0.0), p_long(-32) % p_float(-1.0))
    assertEquals(p_float(-9.0), p_long(-32) % p_float(23.0))
    assertEquals(p_float(-8.877001), p_long(-32) % p_float(23.123))
    assertEquals(p_float(-0.0), p_long(-32) % p_float(-32.0))
    assertEquals(p_float(-32.0), p_long(-32) % p_float(-32.456))
    assertEquals(p_float(-32.0), p_long(-32) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(-32.0), p_long(-32) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(-32) % p_float(Float.NaN_))
    assertEquals(p_float(-32.0), p_long(-32) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(-0.0), p_long(-32) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(Long.MAX_VALUE) % p_float(0.0))
    assertEquals(p_float(0.0), p_long(Long.MAX_VALUE) % p_float(1.0))
    assertEquals(p_float(0.0), p_long(Long.MAX_VALUE) % p_float(-1.0))
    assertEquals(p_float(3.0), p_long(Long.MAX_VALUE) % p_float(23.0))
    assertEquals(p_float(8.53841), p_long(Long.MAX_VALUE) % p_float(23.123))
    assertEquals(p_float(0.0), p_long(Long.MAX_VALUE) % p_float(-32.0))
    assertEquals(p_float(0.9325943), p_long(Long.MAX_VALUE) % p_float(-32.456))
    assertEquals(p_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(Long.MAX_VALUE) % p_float(Float.NaN_))
    assertEquals(p_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(0.0), p_long(Long.MAX_VALUE) % p_float(Float.MIN_VALUE))

    assertEquals(p_float(Float.NaN_), p_long(Long.MIN_VALUE) % p_float(0.0))
    assertEquals(p_float(-0.0), p_long(Long.MIN_VALUE) % p_float(1.0))
    assertEquals(p_float(-0.0), p_long(Long.MIN_VALUE) % p_float(-1.0))
    assertEquals(p_float(-3.0), p_long(Long.MIN_VALUE) % p_float(23.0))
    assertEquals(p_float(-8.53841), p_long(Long.MIN_VALUE) % p_float(23.123))
    assertEquals(p_float(-0.0), p_long(Long.MIN_VALUE) % p_float(-32.0))
    assertEquals(p_float(-0.9325943), p_long(Long.MIN_VALUE) % p_float(-32.456))
    assertEquals(p_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % p_float(Float.POSITIVE_INFINITY))
    assertEquals(p_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % p_float(Float.NEGATIVE_INFINITY))
    assertEquals(p_float(Float.NaN_), p_long(Long.MIN_VALUE) % p_float(Float.NaN_))
    assertEquals(p_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % p_float(Float.MAX_VALUE))
    assertEquals(p_float(-0.0), p_long(Long.MIN_VALUE) % p_float(Float.MIN_VALUE))

    assertEquals(float, statictypeof(p_long(0) % p_float(0.0)))
  }

  function testPLongFloatRemainder() {
    assertEquals(b_float(Float.NaN_), p_long(0) % b_float(0.0))
    assertEquals(b_float(0.0), p_long(0) % b_float(1.0))
    assertEquals(b_float(0.0), p_long(0) % b_float(-1.0))
    assertEquals(b_float(0.0), p_long(0) % b_float(23.0))
    assertEquals(b_float(0.0), p_long(0) % b_float(23.123))
    assertEquals(b_float(0.0), p_long(0) % b_float(-32.0))
    assertEquals(b_float(0.0), p_long(0) % b_float(-32.456))
    assertEquals(b_float(0.0), p_long(0) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(0.0), p_long(0) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(0) % b_float(Float.NaN_))
    assertEquals(b_float(0.0), p_long(0) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), p_long(0) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(1) % b_float(0.0))
    assertEquals(b_float(0.0), p_long(1) % b_float(1.0))
    assertEquals(b_float(0.0), p_long(1) % b_float(-1.0))
    assertEquals(b_float(1.0), p_long(1) % b_float(23.0))
    assertEquals(b_float(1.0), p_long(1) % b_float(23.123))
    assertEquals(b_float(1.0), p_long(1) % b_float(-32.0))
    assertEquals(b_float(1.0), p_long(1) % b_float(-32.456))
    assertEquals(b_float(1.0), p_long(1) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(1.0), p_long(1) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(1) % b_float(Float.NaN_))
    assertEquals(b_float(1.0), p_long(1) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), p_long(1) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(-1) % b_float(0.0))
    assertEquals(b_float(-0.0), p_long(-1) % b_float(1.0))
    assertEquals(b_float(-0.0), p_long(-1) % b_float(-1.0))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(23.0))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(23.123))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(-32.0))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(-32.456))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(-1) % b_float(Float.NaN_))
    assertEquals(b_float(-1.0), p_long(-1) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), p_long(-1) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(23) % b_float(0.0))
    assertEquals(b_float(0.0), p_long(23) % b_float(1.0))
    assertEquals(b_float(0.0), p_long(23) % b_float(-1.0))
    assertEquals(b_float(0.0), p_long(23) % b_float(23.0))
    assertEquals(b_float(23.0), p_long(23) % b_float(23.123))
    assertEquals(b_float(23.0), p_long(23) % b_float(-32.0))
    assertEquals(b_float(23.0), p_long(23) % b_float(-32.456))
    assertEquals(b_float(23.0), p_long(23) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(23.0), p_long(23) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(23) % b_float(Float.NaN_))
    assertEquals(b_float(23.0), p_long(23) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), p_long(23) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(-32) % b_float(0.0))
    assertEquals(b_float(-0.0), p_long(-32) % b_float(1.0))
    assertEquals(b_float(-0.0), p_long(-32) % b_float(-1.0))
    assertEquals(b_float(-9.0), p_long(-32) % b_float(23.0))
    assertEquals(b_float(-8.877001), p_long(-32) % b_float(23.123))
    assertEquals(b_float(-0.0), p_long(-32) % b_float(-32.0))
    assertEquals(b_float(-32.0), p_long(-32) % b_float(-32.456))
    assertEquals(b_float(-32.0), p_long(-32) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(-32.0), p_long(-32) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(-32) % b_float(Float.NaN_))
    assertEquals(b_float(-32.0), p_long(-32) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), p_long(-32) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(Long.MAX_VALUE) % b_float(0.0))
    assertEquals(b_float(0.0), p_long(Long.MAX_VALUE) % b_float(1.0))
    assertEquals(b_float(0.0), p_long(Long.MAX_VALUE) % b_float(-1.0))
    assertEquals(b_float(3.0), p_long(Long.MAX_VALUE) % b_float(23.0))
    assertEquals(b_float(8.53841), p_long(Long.MAX_VALUE) % b_float(23.123))
    assertEquals(b_float(0.0), p_long(Long.MAX_VALUE) % b_float(-32.0))
    assertEquals(b_float(0.9325943), p_long(Long.MAX_VALUE) % b_float(-32.456))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(Long.MAX_VALUE) % b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("9.223372E18")), p_long(Long.MAX_VALUE) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(0.0), p_long(Long.MAX_VALUE) % b_float(Float.MIN_VALUE))

    assertEquals(b_float(Float.NaN_), p_long(Long.MIN_VALUE) % b_float(0.0))
    assertEquals(b_float(-0.0), p_long(Long.MIN_VALUE) % b_float(1.0))
    assertEquals(b_float(-0.0), p_long(Long.MIN_VALUE) % b_float(-1.0))
    assertEquals(b_float(-3.0), p_long(Long.MIN_VALUE) % b_float(23.0))
    assertEquals(b_float(-8.53841), p_long(Long.MIN_VALUE) % b_float(23.123))
    assertEquals(b_float(-0.0), p_long(Long.MIN_VALUE) % b_float(-32.0))
    assertEquals(b_float(-0.9325943), p_long(Long.MIN_VALUE) % b_float(-32.456))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % b_float(Float.POSITIVE_INFINITY))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % b_float(Float.NEGATIVE_INFINITY))
    assertEquals(b_float(Float.NaN_), p_long(Long.MIN_VALUE) % b_float(Float.NaN_))
    assertEquals(b_float(Float.parseFloat("-9.223372E18")), p_long(Long.MIN_VALUE) % b_float(Float.MAX_VALUE))
    assertEquals(b_float(-0.0), p_long(Long.MIN_VALUE) % b_float(Float.MIN_VALUE))

    assertEquals(Float, statictypeof(p_long(0) % b_float(0.0)))
  }

  function testPLongPDoubleRemainder() {
    assertEquals(p_double(Double.NaN_), p_long(0) % p_double(0.0))
    assertEquals(p_double(0.0), p_long(0) % p_double(1.0))
    assertEquals(p_double(0.0), p_long(0) % p_double(-1.0))
    assertEquals(p_double(0.0), p_long(0) % p_double(23.0))
    assertEquals(p_double(0.0), p_long(0) % p_double(23.123))
    assertEquals(p_double(0.0), p_long(0) % p_double(-32.0))
    assertEquals(p_double(0.0), p_long(0) % p_double(-32.456))
    assertEquals(p_double(0.0), p_long(0) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(0.0), p_long(0) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(0) % p_double(Double.NaN_))
    assertEquals(p_double(0.0), p_long(0) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(0.0), p_long(0) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(1) % p_double(0.0))
    assertEquals(p_double(0.0), p_long(1) % p_double(1.0))
    assertEquals(p_double(0.0), p_long(1) % p_double(-1.0))
    assertEquals(p_double(1.0), p_long(1) % p_double(23.0))
    assertEquals(p_double(1.0), p_long(1) % p_double(23.123))
    assertEquals(p_double(1.0), p_long(1) % p_double(-32.0))
    assertEquals(p_double(1.0), p_long(1) % p_double(-32.456))
    assertEquals(p_double(1.0), p_long(1) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(1.0), p_long(1) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(1) % p_double(Double.NaN_))
    assertEquals(p_double(1.0), p_long(1) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(0.0), p_long(1) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(-1) % p_double(0.0))
    assertEquals(p_double(-0.0), p_long(-1) % p_double(1.0))
    assertEquals(p_double(-0.0), p_long(-1) % p_double(-1.0))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(23.0))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(23.123))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(-32.0))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(-32.456))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(-1) % p_double(Double.NaN_))
    assertEquals(p_double(-1.0), p_long(-1) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(-0.0), p_long(-1) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(23) % p_double(0.0))
    assertEquals(p_double(0.0), p_long(23) % p_double(1.0))
    assertEquals(p_double(0.0), p_long(23) % p_double(-1.0))
    assertEquals(p_double(0.0), p_long(23) % p_double(23.0))
    assertEquals(p_double(23.0), p_long(23) % p_double(23.123))
    assertEquals(p_double(23.0), p_long(23) % p_double(-32.0))
    assertEquals(p_double(23.0), p_long(23) % p_double(-32.456))
    assertEquals(p_double(23.0), p_long(23) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(23.0), p_long(23) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(23) % p_double(Double.NaN_))
    assertEquals(p_double(23.0), p_long(23) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(0.0), p_long(23) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(-32) % p_double(0.0))
    assertEquals(p_double(-0.0), p_long(-32) % p_double(1.0))
    assertEquals(p_double(-0.0), p_long(-32) % p_double(-1.0))
    assertEquals(p_double(-9.0), p_long(-32) % p_double(23.0))
    assertEquals(p_double(-8.876999999999999), p_long(-32) % p_double(23.123))
    assertEquals(p_double(-0.0), p_long(-32) % p_double(-32.0))
    assertEquals(p_double(-32.0), p_long(-32) % p_double(-32.456))
    assertEquals(p_double(-32.0), p_long(-32) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(-32.0), p_long(-32) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(-32) % p_double(Double.NaN_))
    assertEquals(p_double(-32.0), p_long(-32) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(-0.0), p_long(-32) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(Long.MAX_VALUE) % p_double(0.0))
    assertEquals(p_double(0.0), p_long(Long.MAX_VALUE) % p_double(1.0))
    assertEquals(p_double(0.0), p_long(Long.MAX_VALUE) % p_double(-1.0))
    assertEquals(p_double(3.0), p_long(Long.MAX_VALUE) % p_double(23.0))
    assertEquals(p_double(10.199447303550599), p_long(Long.MAX_VALUE) % p_double(23.123))
    assertEquals(p_double(0.0), p_long(Long.MAX_VALUE) % p_double(-32.0))
    assertEquals(p_double(3.2543554350506128), p_long(Long.MAX_VALUE) % p_double(-32.456))
    assertEquals(p_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(Long.MAX_VALUE) % p_double(Double.NaN_))
    assertEquals(p_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(0.0), p_long(Long.MAX_VALUE) % p_double(Double.MIN_VALUE))

    assertEquals(p_double(Double.NaN_), p_long(Long.MIN_VALUE) % p_double(0.0))
    assertEquals(p_double(-0.0), p_long(Long.MIN_VALUE) % p_double(1.0))
    assertEquals(p_double(-0.0), p_long(Long.MIN_VALUE) % p_double(-1.0))
    assertEquals(p_double(-3.0), p_long(Long.MIN_VALUE) % p_double(23.0))
    assertEquals(p_double(-10.199447303550599), p_long(Long.MIN_VALUE) % p_double(23.123))
    assertEquals(p_double(-0.0), p_long(Long.MIN_VALUE) % p_double(-32.0))
    assertEquals(p_double(-3.2543554350506128), p_long(Long.MIN_VALUE) % p_double(-32.456))
    assertEquals(p_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % p_double(Double.POSITIVE_INFINITY))
    assertEquals(p_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % p_double(Double.NEGATIVE_INFINITY))
    assertEquals(p_double(Double.NaN_), p_long(Long.MIN_VALUE) % p_double(Double.NaN_))
    assertEquals(p_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % p_double(Double.MAX_VALUE))
    assertEquals(p_double(-0.0), p_long(Long.MIN_VALUE) % p_double(Double.MIN_VALUE))

    assertEquals(double, statictypeof(p_long(0) % p_double(0.0)))
  }

  function testPLongDoubleRemainder() {
    assertEquals(b_double(Double.NaN_), p_long(0) % b_double(0.0))
    assertEquals(b_double(0.0), p_long(0) % b_double(1.0))
    assertEquals(b_double(0.0), p_long(0) % b_double(-1.0))
    assertEquals(b_double(0.0), p_long(0) % b_double(23.0))
    assertEquals(b_double(0.0), p_long(0) % b_double(23.123))
    assertEquals(b_double(0.0), p_long(0) % b_double(-32.0))
    assertEquals(b_double(0.0), p_long(0) % b_double(-32.456))
    assertEquals(b_double(0.0), p_long(0) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(0.0), p_long(0) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(0) % b_double(Double.NaN_))
    assertEquals(b_double(0.0), p_long(0) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), p_long(0) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(1) % b_double(0.0))
    assertEquals(b_double(0.0), p_long(1) % b_double(1.0))
    assertEquals(b_double(0.0), p_long(1) % b_double(-1.0))
    assertEquals(b_double(1.0), p_long(1) % b_double(23.0))
    assertEquals(b_double(1.0), p_long(1) % b_double(23.123))
    assertEquals(b_double(1.0), p_long(1) % b_double(-32.0))
    assertEquals(b_double(1.0), p_long(1) % b_double(-32.456))
    assertEquals(b_double(1.0), p_long(1) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(1.0), p_long(1) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(1) % b_double(Double.NaN_))
    assertEquals(b_double(1.0), p_long(1) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), p_long(1) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(-1) % b_double(0.0))
    assertEquals(b_double(-0.0), p_long(-1) % b_double(1.0))
    assertEquals(b_double(-0.0), p_long(-1) % b_double(-1.0))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(23.0))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(23.123))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(-32.0))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(-32.456))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(-1) % b_double(Double.NaN_))
    assertEquals(b_double(-1.0), p_long(-1) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), p_long(-1) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(23) % b_double(0.0))
    assertEquals(b_double(0.0), p_long(23) % b_double(1.0))
    assertEquals(b_double(0.0), p_long(23) % b_double(-1.0))
    assertEquals(b_double(0.0), p_long(23) % b_double(23.0))
    assertEquals(b_double(23.0), p_long(23) % b_double(23.123))
    assertEquals(b_double(23.0), p_long(23) % b_double(-32.0))
    assertEquals(b_double(23.0), p_long(23) % b_double(-32.456))
    assertEquals(b_double(23.0), p_long(23) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(23.0), p_long(23) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(23) % b_double(Double.NaN_))
    assertEquals(b_double(23.0), p_long(23) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), p_long(23) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(-32) % b_double(0.0))
    assertEquals(b_double(-0.0), p_long(-32) % b_double(1.0))
    assertEquals(b_double(-0.0), p_long(-32) % b_double(-1.0))
    assertEquals(b_double(-9.0), p_long(-32) % b_double(23.0))
    assertEquals(b_double(-8.876999999999999), p_long(-32) % b_double(23.123))
    assertEquals(b_double(-0.0), p_long(-32) % b_double(-32.0))
    assertEquals(b_double(-32.0), p_long(-32) % b_double(-32.456))
    assertEquals(b_double(-32.0), p_long(-32) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(-32.0), p_long(-32) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(-32) % b_double(Double.NaN_))
    assertEquals(b_double(-32.0), p_long(-32) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), p_long(-32) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(Long.MAX_VALUE) % b_double(0.0))
    assertEquals(b_double(0.0), p_long(Long.MAX_VALUE) % b_double(1.0))
    assertEquals(b_double(0.0), p_long(Long.MAX_VALUE) % b_double(-1.0))
    assertEquals(b_double(3.0), p_long(Long.MAX_VALUE) % b_double(23.0))
    assertEquals(b_double(10.199447303550599), p_long(Long.MAX_VALUE) % b_double(23.123))
    assertEquals(b_double(0.0), p_long(Long.MAX_VALUE) % b_double(-32.0))
    assertEquals(b_double(3.2543554350506128), p_long(Long.MAX_VALUE) % b_double(-32.456))
    assertEquals(b_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(Long.MAX_VALUE) % b_double(Double.NaN_))
    assertEquals(b_double(Double.parseDouble("9.223372036854776E18")), p_long(Long.MAX_VALUE) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(0.0), p_long(Long.MAX_VALUE) % b_double(Double.MIN_VALUE))

    assertEquals(b_double(Double.NaN_), p_long(Long.MIN_VALUE) % b_double(0.0))
    assertEquals(b_double(-0.0), p_long(Long.MIN_VALUE) % b_double(1.0))
    assertEquals(b_double(-0.0), p_long(Long.MIN_VALUE) % b_double(-1.0))
    assertEquals(b_double(-3.0), p_long(Long.MIN_VALUE) % b_double(23.0))
    assertEquals(b_double(-10.199447303550599), p_long(Long.MIN_VALUE) % b_double(23.123))
    assertEquals(b_double(-0.0), p_long(Long.MIN_VALUE) % b_double(-32.0))
    assertEquals(b_double(-3.2543554350506128), p_long(Long.MIN_VALUE) % b_double(-32.456))
    assertEquals(b_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % b_double(Double.POSITIVE_INFINITY))
    assertEquals(b_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % b_double(Double.NEGATIVE_INFINITY))
    assertEquals(b_double(Double.NaN_), p_long(Long.MIN_VALUE) % b_double(Double.NaN_))
    assertEquals(b_double(Double.parseDouble("-9.223372036854776E18")), p_long(Long.MIN_VALUE) % b_double(Double.MAX_VALUE))
    assertEquals(b_double(-0.0), p_long(Long.MIN_VALUE) % b_double(Double.MIN_VALUE))

    assertEquals(Double, statictypeof(p_long(0) % b_double(0.0)))
  }

  function testPLongBigIntegerRemainder() {
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(0) % big_int("0"))
    assertEquals(big_int("0"), p_long(0) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(0) % big_int("-1"))
    assertEquals(big_int("0"), p_long(0) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(0) % big_int("-32"))
    assertEquals(big_int("0"), p_long(0) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(0) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(1) % big_int("0"))
    assertEquals(big_int("0"), p_long(1) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(1) % big_int("-1"))
    assertEquals(big_int("1"), p_long(1) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(1) % big_int("-32"))
    assertEquals(big_int("1"), p_long(1) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(1) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-1) % big_int("0"))
    assertEquals(big_int("0"), p_long(-1) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-1) % big_int("-1"))
    assertEquals(big_int("22"), p_long(-1) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-1) % big_int("-32"))
    assertEquals(big_int("123456789012345678901234567889"), p_long(-1) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-1) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(23) % big_int("0"))
    assertEquals(big_int("0"), p_long(23) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(23) % big_int("-1"))
    assertEquals(big_int("0"), p_long(23) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(23) % big_int("-32"))
    assertEquals(big_int("23"), p_long(23) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(23) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-32) % big_int("0"))
    assertEquals(big_int("0"), p_long(-32) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-32) % big_int("-1"))
    assertEquals(big_int("14"), p_long(-32) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-32) % big_int("-32"))
    assertEquals(big_int("123456789012345678901234567858"), p_long(-32) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(-32) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MAX_VALUE) % big_int("0"))
    assertEquals(big_int("0"), p_long(Long.MAX_VALUE) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MAX_VALUE) % big_int("-1"))
    assertEquals(big_int("2"), p_long(Long.MAX_VALUE) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MAX_VALUE) % big_int("-32"))
    assertEquals(big_int("9223372036854775807"), p_long(Long.MAX_VALUE) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MAX_VALUE) % big_int("-123456789012345678901234567890"))

    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MIN_VALUE) % big_int("0"))
    assertEquals(big_int("0"), p_long(Long.MIN_VALUE) % big_int("1"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MIN_VALUE) % big_int("-1"))
    assertEquals(big_int("20"), p_long(Long.MIN_VALUE) % big_int("23"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MIN_VALUE) % big_int("-32"))
    assertEquals(big_int("123456789003122306864379792082"), p_long(Long.MIN_VALUE) % big_int("123456789012345678901234567890"))
    assertThrowsArithmeticException("BigInteger: modulus not positive", \ -> p_long(Long.MIN_VALUE) % big_int("-123456789012345678901234567890"))

    assertEquals(BigInteger, statictypeof(p_long(0) % big_int("0")))
  }

  function testPLongBigDecimalRemainder() {
    assertThrowsArithmeticException("Division undefined", \ -> p_long(0) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("-1"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("23"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("-32"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("-32.456"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("0"), p_long(0) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(1) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(1) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(1) % big_decimal("-1"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("23"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("23.123"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("-32"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("-32.456"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("1"), p_long(1) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(-1) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(-1) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(-1) % big_decimal("-1"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("23"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("23.123"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("-32"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("-32.456"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("1"), p_long(-1) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(23) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(23) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(23) % big_decimal("-1"))
    assertEquals(big_decimal("0"), p_long(23) % big_decimal("23"))
    assertEquals(big_decimal("23"), p_long(23) % big_decimal("23.123"))
    assertEquals(big_decimal("23"), p_long(23) % big_decimal("-32"))
    assertEquals(big_decimal("23"), p_long(23) % big_decimal("-32.456"))
    assertEquals(big_decimal("23"), p_long(23) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("23"), p_long(23) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(-32) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(-32) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(-32) % big_decimal("-1"))
    assertEquals(big_decimal("9"), p_long(-32) % big_decimal("23"))
    assertEquals(big_decimal("8.877"), p_long(-32) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), p_long(-32) % big_decimal("-32"))
    assertEquals(big_decimal("32"), p_long(-32) % big_decimal("-32.456"))
    assertEquals(big_decimal("32"), p_long(-32) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("32"), p_long(-32) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(Long.MAX_VALUE) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(Long.MAX_VALUE) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(Long.MAX_VALUE) % big_decimal("-1"))
    assertEquals(big_decimal("2"), p_long(Long.MAX_VALUE) % big_decimal("23"))
    assertEquals(big_decimal("12.003"), p_long(Long.MAX_VALUE) % big_decimal("23.123"))
    assertEquals(big_decimal("31"), p_long(Long.MAX_VALUE) % big_decimal("-32"))
    assertEquals(big_decimal("30.704"), p_long(Long.MAX_VALUE) % big_decimal("-32.456"))
    assertEquals(big_decimal("9223372036854775807"), p_long(Long.MAX_VALUE) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("9223372036854775807"), p_long(Long.MAX_VALUE) % big_decimal("-123456789012345678901234567890.123456789"))

    assertThrowsArithmeticException("Division by zero", \ -> p_long(Long.MIN_VALUE) % big_decimal("0"))
    assertEquals(big_decimal("0"), p_long(Long.MIN_VALUE) % big_decimal("1"))
    assertEquals(big_decimal("0"), p_long(Long.MIN_VALUE) % big_decimal("-1"))
    assertEquals(big_decimal("3"), p_long(Long.MIN_VALUE) % big_decimal("23"))
    assertEquals(big_decimal("13.003"), p_long(Long.MIN_VALUE) % big_decimal("23.123"))
    assertEquals(big_decimal("0"), p_long(Long.MIN_VALUE) % big_decimal("-32"))
    assertEquals(big_decimal("31.704"), p_long(Long.MIN_VALUE) % big_decimal("-32.456"))
    assertEquals(big_decimal("9223372036854775808"), p_long(Long.MIN_VALUE) % big_decimal("123456789012345678901234567890.123456789"))
    assertEquals(big_decimal("9223372036854775808"), p_long(Long.MIN_VALUE) % big_decimal("-123456789012345678901234567890.123456789"))

    assertEquals(BigDecimal, statictypeof(p_long(0) % big_decimal("0")))
  }

}

